Package: signatureSearchData
Title: Datasets for signatureSearch package
Description: CMAP/LINCS hdf5 databases and other annotations used for 
             signatureSearch software package.
Version: 1.27.0
Author: Yuzhu Duan <yduan004@ucr.edu>,
        Brendan Gongol <bgong001@ucr.edu>,
        Thomas Girke <thomas.girke@ucr.edu>
Maintainer: Brendan Gongol <bgong001@ucr.edu>
Depends: R (>= 3.6)
Imports: ExperimentHub, utils, affy, limma, Biobase, magrittr, dplyr,
        R.utils, stats, rhdf5
Suggests: knitr, rmarkdown
License: Artistic-2.0
biocViews: ExperimentHub, ExperimentData, ExpressionData
Encoding: UTF-8
VignetteBuilder: knitr
RoxygenNote: 7.1.2
git_url: https://git.bioconductor.org/packages/signatureSearchData
git_branch: devel
git_last_commit: a88e9f7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-30
NeedsCompilation: no
Packaged: 2026-04-30 15:59:52 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(combineNormRes)
export(combineResults)
export(getCmapCEL)
export(inst_filter)
export(meanExpr)
export(normalizeCel)
export(probe2gene)
export(runLimma)
export(sampleList)
export(sig_filter)
importFrom(Biobase,ExpressionSet)
importFrom(R.utils,bunzip2)
importFrom(affy,ReadAffy)
importFrom(affy,exprs)
importFrom(affy,mas5)
importFrom(dplyr,bind_cols)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,makeContrasts)
importFrom(limma,topTable)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(stats,model.matrix)
importFrom(utils,download.file)
importFrom(utils,read.delim)
importFrom(utils,unzip)
importFrom(utils,write.table)

# _signatureSearchData_: Reference Data for Gene Expression Signature Searching

# Introduction

The `signatureSearchData` package provides access to the reference data used by
the associated `signatureSearch` software package. The latter allows to search 
with a query gene expression signature (GES) a database of treatment GESs to 
identify cellular states sharing similar expression responses (connections). This 
way one can identify drugs or gene knockouts that induce expression phenotypes 
similar to a sample of interest. The resulting associations may lead to novel 
functional insights how perturbagens of interest interact with biological systems. 

Currently, `signatureSearchData` includes GES data from the CMap (Connectivity
Map) and LINCS (Library of Network-Based Cellular Signatures) projects that are
largely based on drug and genetic perturbation experiments performed on
variable numbers of human cell lines [@Lamb2006-du; @Subramanian2017-fu]. In
`signatureSearchData` these data sets have been preprocessed to be compatible
with the different gene expression signature search (GESS) algorithms
implemented in `signatureSearch`. The preprocessed data types include but are
not limited to normalized gene expression values (_e.g._ intensity values), log
fold changes (LFC) and Z-scores, p-values or FDRs of differentially expressed
genes (DEGs), rankings based on selected preprocessing routines or sets of top
up/down-regulated DEGs. 

The CMap data were downloaded from the [CMap project
site](https://portals.broadinstitute.org/cmap/) (Version build02). The latter is
a collection of over 7,000 gene expression profiles (signatures) obtained
from perturbation experiments with 1,309 drug-like small molecules on five
human cancer cell lines. The Affymetrix Gene Chip technology was used to
generate the CMAP2 data set. 

In 2017, the LINCS Consortium generated a similar but much larger data set where
the total number of gene expression signatures was scaled up to over one
million. This was achieved by switching to a much more cost effective gene
expression profiling technology called L1000 assay [@Peck2006-rf;
@Edgar2002-di]. The current set of perturbations covered by the LINCS data set
includes 19,811 drug-like small molecules applied at variable concentrations
and treatment times to ~70 human non-cancer (normal) and cancer cell lines.
Additionally, it includes several thousand genetic perturbagens composed of
gene knockdown and over-expression experiments. 

In 2020, the LINCS 2017 database is expanded to the beta release,
here refer to as LINCS2. It contains >80k perturbations and >200 cell lines and 
over 3M gene expression profiles. This represents roughly a 3-fold expansion on 
the LINCS 2017 database and notable new subsets of data include CRISPSR knockout 
of >5k genes and hematopoietic and non-cancer cell models. 
The datasets can be accessed at https://clue.io/releases/data-dashboard.

The data structures and search algorithms used by `signatureSearch` and
`signatureSearchData` are designed to work with most genome-wide expression
data including hybridization-based methods, such as Affymetrix or L1000, as
well as sequencing-based methods, such as RNA-Seq. Currently,
`signatureSearchData` does not include preconfigured RNA-Seq reference data mainly 
due to the lack of large-scale perturbation studies (_e.g._ drug-based) available 
in the public domain that are based on RNA-Seq. This situation may change in 
the near future once the technology has become more affordable for this purpose. 

# Vignette
The vignette of this package is available at [here](https://www.bioconductor.org/packages/release/data/experiment/vignettes/signatureSearchData/inst/doc/signatureSearchData.html)

# Install and Load Package

`signatureSearchData` is a R/Bioconductor package and can be installed using 
`BiocManager::install()`.

```r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("signatureSearchData", version = "3.9")
```

To obtain the most recent updates immediately, one can install it directly from GitHub as follows.
```r
devtools::install_github("yduan004/signatureSearchData")
```
After the package is installed, it can be loaded into an R session as follows.
```r
library(signatureSearchData)
```
For a detailed description of loading and/or generating signature databases,
please refer to the vignette of this package by running
`browseVignettes("signatureSearchData")` in an R session.

normalizeCel <- function(chiptype_list, rerun = FALSE){
  mydir <- getwd()
  if(rerun==TRUE){
    for(i in names(chiptype_list)){
      dir.create(paste0("./data/",i))
      setwd(paste0("./data/",i))
      mydata <- affy::ReadAffy(filenames=chiptype_list[[i]], 
                               celfile.path="../CEL")
      eset <- affy::rma(mydata)
      write.table(affy::exprs(eset), file="./rma_exprs.xls", 
                  quote=FALSE, sep="\t",col.names=NA)
      setwd("../../")
    }
  } else {
    print("To execute function, set 'rerun=TRUE'")
  }
  setwd(mydir)
}

list2link <- function(list, type){
  link <- data.frame()
  for(i in 1:length(list)){
    if(length(list[[i]])!=0){
      if(type=="dt"){
        tmp <- data.frame(drug_name=names(list[i]), 
                          t_gn_sym=as.character(list[[i]]), 
                          stringsAsFactors = FALSE)
      }
      if(type=="td"){
        tmp <- data.frame(t_gn_sym=names(list[i]), 
                          drug_name=as.character(list[[i]]), 
                          stringsAsFactors = FALSE)
      }
      link <- rbind(link, tmp)
    }
  }
  return(link)
}

getPertInfo <- function(pert_iname){
  pert_info = NULL
  col = c("canonical_smiles", "description", "inchi_key", "inchi_string", 
          "moa", "molecular_formula", "pert_id", "pert_iname", "pert_type", 
          "pubchem_cid", "target")
  for(iname in pert_iname){
    json <- system(paste0(
                 'curl -X GET --globoff --header "Accept: application/json" --header "user_key: 6e09440bcbfa67ebfac72b928d4b9366" "https://api.clue.io/api/perts?filter=%7B%22where%22%3A%7B%22pert_iname%22%3A%22',
                  iname, '%22%7D%7D"'),
                  intern = TRUE, ignore.stderr = TRUE)
    
    if(length(json)==0){
      message("Curl error happens on ", iname)
      next()
    }
    if(grepl("error",json) | json=="[]"){
      message("API error happens on ", iname)
      next()
    }
    require(jsonlite)
    jsonr <- fromJSON(json)
    # print(i)
    # i=i+1
    # rownames(jsonr) <- pert_id
    for(item in col){
      if(! item %in% colnames(jsonr)){
        new <- data.frame(NA)
        colnames(new) <- item
        jsonr = cbind(jsonr, new)
      }
    }
    pert_info <- rbind(pert_info, jsonr[,col])
  }
  return(pert_info)
}

#' Filter LINCS Level 5 by Condition
#' 
#' Function to filter Level 5 data from LINCS by specific conditions, such as compound 
#' concentrations and treatment times.
#' @param meta tibble containing experimental conditions of LINCS data
#' @param pert_type perturbation type, 'trt_cp' refers to treatment ('trt') with compound ('cp'). 
#' Description of other perturbation types ('pert_type') can be found in the GEO CMap LINCS 
#' User Guide v2.1 URL: https://docs.google.com/document/d/1rbHBy3DKekFm9lZouRG-ZcfLmCsfkUKzGPxxjqxPlYw/edit#
#' @param dose concentration of compound used for treatment, needs to match elements
#' in 'pert_idose' column of 'meta'
#' @param time compound treatment time, needs to match elements in 'pert_itime' 
#' column of 'meta'
#' @return tibble
#' @importFrom magrittr %<>%
#' @importFrom magrittr %>%
#' @importFrom dplyr filter
#' @importFrom dplyr bind_cols
#' @importFrom dplyr distinct
#' @examples 
#' meta <- data.frame(pert_idose=c("2 um","4 um", "10 um"), 
#'                    pert_itime="24 h",
#'                    pert_type="trt_cp", pert_iname=c("p1","p2","p3"),
#'                    cell_id="MCF7")
#' sig_filter(meta, dose="2 um")
#' @export
sig_filter <- function(meta, pert_type="trt_cp", dose, time="24 h"){
    meta %<>% dplyr::filter(pert_type==pert_type & pert_idose==dose & pert_itime==time) 
    meta %<>% 
        bind_cols(alt_id=paste(meta$pert_iname, meta$cell_id, sep="_")) %>%
        bind_cols(pert_cell_factor=paste(meta$pert_iname, meta$cell_id, 
                                         meta$pert_type, sep="__")) %>% 
        distinct(alt_id, .keep_all=TRUE) # eliminate technical duplicates
    return(meta)
}

#' Filter LINCS Level 3 by Condition
#' 
#' Function to filter Level 3 data from LINCS by specific conditions, such as compound 
#' concentrations and treatment times.
#' @param meta tibble containing experimental conditions of LINCS data
#' @param pert_type perturbation type, 'trt_cp' refers to treatment ('trt') with 
#' compound ('cp'). Description of other perturbation types ('pert_type') can 
#' be found in the GEO CMap LINCS User Guide v2.1 
#' URL: https://docs.google.com/document/d/1rbHBy3DKekFm9lZouRG-ZcfLmCsfkUKzGPxxjqxPlYw/edit#
#' @param dose concentration of compound used for treatment, needs to match elements
#' in 'pert_dose' column of 'meta'
#' @param dose_unit unit of dose of compound treatment, needs to match elements
#' in 'pert_dose_unit' column of 'meta'
#' @param time compound treatment time, needs to match elements in 'pert_time' 
#' column of 'meta'
#' @param time_unit unit of time, needs to match elements in 'pert_time_unit' 
#' column of 'meta'
#' @return tibble
#' @examples 
#' meta <- data.frame(pert_dose=c(2,4,10), pert_dose_unit="um", 
#'                    pert_time=24, pert_time_unit="h",
#'                    pert_type="trt_cp", pert_iname=c("p1","p2","p3"),
#'                    cell_id="MCF7")
#' inst_filter(meta)
#' @export
inst_filter <- function(meta, pert_type="trt_cp", dose=10, dose_unit="um", time=24, time_unit="h"){
    meta %<>% dplyr::filter(pert_type==pert_type & pert_dose==dose & pert_dose_unit== dose_unit
                       & pert_time==time & pert_time_unit==time_unit)
    meta %<>% bind_cols(alt_id=paste(meta$pert_iname, meta$cell_id, sep="_")) %>% 
        bind_cols(pert_cell_factor=paste(meta$pert_iname, meta$cell_id, meta$pert_type, sep="__"))
    return(meta)
}

alt_id = pert_dose = pert_dose_unit = pert_idose = pert_itime =
pert_time = pert_time_unit = pert_type = NULL

## ----setup, echo=FALSE, messages=FALSE, warnings=FALSE------------------------
suppressPackageStartupMessages({
    library(signatureSearchData)
})
# knitr::opts_knit$set(root.dir = "~/insync/project/longevityTools_eDRUG/")

## ----install, eval=FALSE------------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# BiocManager::install("signatureSearchData")

## ----load, eval=FALSE---------------------------------------------------------
# library(signatureSearchData)

## ----eh_explore_ssd1, eval=TRUE, warning=FALSE, message=FALSE-----------------
library(ExperimentHub)
eh <- ExperimentHub()
ssd <- query(eh, c("signatureSearchData"))
ssd

## ----eh_explore_ssd2, eval=TRUE, warning=FALSE, message=FALSE-----------------
ssd$title

## ----eh_explore_ssd3, eval=TRUE-----------------------------------------------
as.list(ssd)[10]

## ----lincs, eval=FALSE--------------------------------------------------------
# library(ExperimentHub); library(rhdf5)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "lincs"))
# lincs_path <- eh[['EH3226']]
# rhdf5::h5ls(lincs_path)

## ----filter_meta42, eval=FALSE------------------------------------------------
# meta42 <- readr::read_tsv("./data/GSE92742_Broad_LINCS_sig_info.txt")
# dose <- meta42$pert_idose[7]
# ## filter rows by 'pert_type' as compound, 10uM concentration, and 24h treatment time
# meta42_filter <- sig_filter(meta42, pert_type="trt_cp", dose=dose, time="24 h") # 45956 X 14

## ----extract_modz, eval=FALSE-------------------------------------------------
# library(signatureSearch)
# gctx <- "./data/GSE92742_Broad_LINCS_Level5_COMPZ.MODZ_n473647x12328.gctx"
# gctx2h5(gctx, cid=meta42_filter$sig_id, new_cid=meta42_filter$pert_cell_factor,
#         h5file="./data/lincs.h5", by_ncol=5000, overwrite=TRUE)
# library(HDF5Array)
# se <- SummarizedExperiment(HDF5Array("./data/lincs.h5", name="assay"))
# rownames(se) <- HDF5Array("./data/lincs.h5", name="rownames")
# colnames(se) <- HDF5Array("./data/lincs.h5", name="colnames")

## ----lincs_degs, eval=FALSE---------------------------------------------------
# library(signatureSearch)
# # Get up and down 150 DEGs
# degs <- getDEGSig(cmp="vorinostat", cell="SKB", refdb="lincs", Nup=150, Ndown=150)
# 
# # Get DEGs by setting cutoffs
# degs2 <- getDEGSig(cmp="vorinostat", cell="SKB", refdb="lincs", higher=2, lower=-2)

## ----lincs_degs_db, eval=FALSE------------------------------------------------
# # gCMAP method
# gep <- getSig("vorinostat", "SKB", refdb="lincs")
# qsig_gcmap <- qSig(query = gep, gess_method = "gCMAP", refdb = "lincs")
# gcmap_res <- gess_gcmap(qsig_gcmap, higher=2, lower=-2)
# # Fisher method
# qsig_fisher <- qSig(query = degs, gess_method = "Fisher", refdb = "lincs")
# fisher_res <- gess_fisher(qSig=qsig_fisher, higher=2, lower=-2)

## ----lincs_expr, eval=FALSE---------------------------------------------------
# library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "lincs_expr"))
# lincs_expr_path <- eh[['EH3227']]

## ----filter_expr, eval=FALSE--------------------------------------------------
# inst42 <- readr::read_tsv("./data/GSE92742_Broad_LINCS_inst_info.txt")
# inst42_filter <- inst_filter(inst42, pert_type="trt_cp", dose=10, dose_unit="um",
#                              time=24, time_unit="h") # 169,795 X 13

## ----extract_expr, eval=FALSE-------------------------------------------------
# # It takes some time
# library(signatureSearch)
# meanExpr2h5(gctx="./data/GSE92742_Broad_LINCS_Level3_INF_mlr12k_n1319138x12328.gctx",
#             inst=inst42_filter, h5file="./data/lincs_expr.h5") # 12328 X 38824

## ----lincs2, eval=FALSE-------------------------------------------------------
# library(ExperimentHub); library(rhdf5)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "lincs2"))
# lincs2_path <- eh[['EH7297']]
# rhdf5::h5ls(lincs2_path)

## ----lincs2_hdf5, eval=FALSE--------------------------------------------------
# siginfo_beta <- fread("https://s3.amazonaws.com/macchiato.clue.io/builds/LINCS2020/siginfo_beta.txt")
# exemplar <- siginfo_beta %>% filter(pert_type=="trt_cp" & is_exemplar_sig == 1)
# new_cid <- paste(exemplar$pert_id, exemplar$cell_iname, rep("trt_cp", length(exemplar$cmap_name)), sep="__")
# gctx2h5("level5_beta_trt_cp_n720216x12328.gctx", cid=exemplar$sig_id, new_cid=new_cid,
#         h5file="lincs2.h5", by_ncol=5000, overwrite=TRUE)
# DBpath <- "lincs2.h5"
# sedb <- SummarizedExperiment(HDF5Array(DBpath, name="assay"))
# rownames(sedb) <- HDF5Array(DBpath, name="rownames")
# colnames(sedb) <- HDF5Array(DBpath, name="colnames")

## ----cmap_rank, eval=FALSE----------------------------------------------------
# library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "cmap_rank"))
# cmap_rank_path <- eh[["EH3225"]]
# se <- SummarizedExperiment(HDF5Array(cmap_rank_path, name="assay"))
# rownames(se) <- HDF5Array(cmap_rank_path, name="rownames")
# colnames(se) <- HDF5Array(cmap_rank_path, name="colnames")

## ----filter_rankm, eval=FALSE-------------------------------------------------
# path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
# cmap_inst <- read.delim(path, check.names=FALSE)
# inst_id <- cmap_inst$instance_id[!duplicated(paste(cmap_inst$cmap_name, cmap_inst$cell2, sep="_"))]
# rankM <- read.delim("./data/rankMatrix.txt", check.names=FALSE, row.names=1) # 22283 X 6100
# rankM_sub <- rankM[, as.character(inst_id)]
# colnames(rankM_sub) <- unique(paste(cmap_inst$cmap_name, cmap_inst$cell2, "trt_cp", sep="__"))

## ----affyid_annot, eval=FALSE, message=FALSE----------------------------------
# library(hgu133a.db)
# myAnnot <- data.frame(ACCNUM=sapply(contents(hgu133aACCNUM), paste, collapse=", "),
#                       SYMBOL=sapply(contents(hgu133aSYMBOL), paste, collapse=", "),
#                       UNIGENE=sapply(contents(hgu133aUNIGENE), paste, collapse=", "),
#                       ENTREZID=sapply(contents(hgu133aENTREZID), paste, collapse=", "),
#                       ENSEMBL=sapply(contents(hgu133aENSEMBL), paste, collapse=", "),
#                       DESC=sapply(contents(hgu133aGENENAME), paste, collapse=", "))
# saveRDS(myAnnot, "./data/myAnnot.rds")

## ----mr_prob, eval=FALSE------------------------------------------------------
# rankM_sub_gene <- probe2gene(rankM_sub, myAnnot)

## ----cmap_rank2h5, eval=FALSE-------------------------------------------------
# matrix2h5(rankM_sub_gene, "./data/cmap_rank.h5", overwrite=TRUE) # 12403 X 3587
# rhdf5::h5ls("./data/cmap_rank.h5")
# ## Read in cmap_rank.h5 as SummarizedExperiment object
# se <- SummarizedExperiment(HDF5Array("./data/cmap_rank.h5", name="assay"))
# rownames(se) <- HDF5Array("./data/cmap_rank.h5", name="rownames")
# colnames(se) <- HDF5Array("./data/cmap_rank.h5", name="colnames")

## ----cmap_expr, eval=FALSE----------------------------------------------------
# library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "cmap_expr"))
# cmap_expr_path <- eh[["EH3224"]]

## ----work_envir, eval=FALSE---------------------------------------------------
# dir.create("data"); dir.create("data/CEL"); dir.create("results")

## ----download_cmap, eval=FALSE------------------------------------------------
# getCmapCEL(rerun=FALSE)

## ----get_cel_type, eval=FALSE-------------------------------------------------
# library(affxparser)
# celfiles <- list.files("./data/CEL", pattern=".CEL$")
# chiptype <- sapply(celfiles, function(x) affxparser::readCelHeader(paste0("data/CEL/", x))$chiptype)
# saveRDS(chiptype, "./data/chiptype.rds")

## ----normalize_chips, eval=FALSE----------------------------------------------
# chiptype <- readRDS("./data/chiptype.rds")
# chiptype_list <- split(names(chiptype), as.character(chiptype))
# normalizeCel(chiptype_list, batchsize=200, rerun=FALSE)

## ----comb_chip_type_data, eval=FALSE------------------------------------------
# chiptype_dir <- unique(chiptype)
# combineResults(chiptype_dir, rerun=FALSE)
# mas5df <- combineNormRes(chiptype_dir, norm_method="MAS5")

## ----prof2gene, eval=FALSE----------------------------------------------------
# myAnnot <- readRDS("./data/myAnnot.rds")
# mas5df <- probe2gene(mas5df, myAnnot)
# saveRDS(mas5df,"./data/mas5df.rds")

## ----rma2cmap_expr, eval=FALSE------------------------------------------------
# mas5df <- readRDS("./data/mas5df.rds") # dim: 12403 x 7056
# path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
# cmap_inst <- read.delim(path, check.names=FALSE)
# cmap_drug_cell_expr <- meanExpr(mas5df, cmap_inst) # dim: 12403 X 3587
# saveRDS(cmap_drug_cell_expr, "./data/cmap_drug_cell_expr.rds")

## ----gen_cmap_expr, eval=FALSE------------------------------------------------
# cmap_drug_cell_expr <- readRDS("./data/cmap_drug_cell_expr.rds")
# ## match colnames to '(drug)__(cell)__(factor)' format
# colnames(cmap_drug_cell_expr) <- gsub("(^.*)_(.*$)", "\\1__\\2__trt_cp",
#                                       colnames(cmap_drug_cell_expr))
# matrix2h5(cmap_drug_cell_expr, "./data/cmap_expr.h5", overwrite=TRUE)
# h5ls("./data/cmap_expr.h5")

## ----cmap, eval=FALSE---------------------------------------------------------
# library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "cmap"))
# cmap_path <- eh[["EH3223"]]

## ----cel_file_list, eval=FALSE------------------------------------------------
# path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
# cmap_inst <- read.delim(path, check.names=FALSE)
# comp_list <- sampleList(cmap_inst, myby="CMP_CELL")

## ----deg_limma, eval=FALSE----------------------------------------------------
# mas5df <- readRDS("./data/mas5df.rds")
# degList <- runLimma(df=log2(mas5df), comp_list, fdr=0.10, foldchange=1, verbose=TRUE)
# saveRDS(degList, "./results/degList.rds") # saves entire degList

## ----se, eval=FALSE-----------------------------------------------------------
# degList <- readRDS("./results/degList.rds")
# logMA <- degList$logFC
# ## match colnames of logMA to '(drug)__(cell)__(factor)' format
# colnames(logMA) <- gsub("(^.*)_(.*$)", "\\1__\\2__trt_cp", colnames(logMA))
# fdr <- degList$FDR
# colnames(fdr) <- gsub("(^.*)_(.*$)", "\\1__\\2__trt_cp", colnames(fdr))
# matrix2h5(logMA, "./data/cmap.h5", name="assay", overwrite=TRUE) # 12403 X 3478
# matrix2h5(fdr, "./data/cmap.h5", name="padj", overwrite=FALSE)
# rhdf5::h5ls("./data/cmap.h5")

## ----cmap_degs, eval=FALSE----------------------------------------------------
# library(signatureSearch)
# # Get up and down 150 DEGs
# degs <- getDEGSig(cmp="vorinostat", cell="PC3", refdb="cmap", Nup=150, Ndown=150)
# 
# # Get DEGs by setting cutoffs
# degs2 <- getDEGSig(cmp="vorinostat", cell="PC3", refdb="cmap",
#                  higher=1, lower=-1, padj=0.05)

## ----cmap_degs_db, eval=FALSE-------------------------------------------------
# # gCMAP method
# gep <- getSig("vorinostat", "PC3", refdb="cmap")
# qsig_gcmap <- qSig(query = gep, gess_method = "gCMAP", refdb = "cmap")
# gcmap_res <- gess_gcmap(qsig_gcmap, higher=1, lower=-1, padj=0.05)
# # Fisher method
# qsig_fisher <- qSig(query = degs, gess_method = "Fisher", refdb = "cmap")
# fisher_res <- gess_fisher(qSig=qsig_fisher, higher=1, lower=-1, padj=0.05)

## ----sessionInfo--------------------------------------------------------------
sessionInfo()

#' This function will download the 7,056 CEL files from the CMap2 project 
#' site (http://www.broadinstitute.org/cmap), and save each of them to a
#' subdirectory named CEL under data. Since this download step will take 
#' a long time, the \code{rerun} argument has been assigned \code{FALSE} in
#' the example code below to avoid running it accidentally. If the raw data are not
#' needed, users can skip this time consuming step and work with the preprocessed
#' \code{\link{cmap}} or \code{\link{cmap_expr}} database downloaded from the 
#' ExperimentHub instead.
#' @title Download CMap2 CEL Files
#' @param rerun TRUE or FALSE, whether to download the data
#' @param dest_dir character(1), path to the destination directory
#' @return download files
#' @examples 
#' getCmapCEL(rerun=FALSE) # set 'rerun' to TRUE if download
#' @importFrom R.utils bunzip2
#' @importFrom utils download.file
#' @importFrom utils unzip
#' @export
getCmapCEL <- function(rerun=TRUE, dest_dir=tempdir()) {
    if(rerun) {
        ## Check for presence of target directory
        if(!dir.exists(dest_dir))
            stop(paste0("Target directory ", dest_dir, " does not exist. 
             Create it to run function."))
        dest_cel <- file.path(dest_dir, "CEL")
        dir.create(dest_cel)
        ## Download CEL files
        for(vol in 1:7){
            dest_vol <- paste0(dest_cel, "/cmap_build02.volume", vol, "of7.zip")
            download.file(paste0("ftp://ftp.broad.mit.edu/pub/cmap/cmap_build02.volume", vol, "of7.zip"), 
                          dest_vol)
            unzip(dest_vol, exdir=dest_cel)
            unlink(dest_vol)
        }
        
        ## Uncompress CEL files
        myfiles <- list.files(dest_cel, pattern=".CEL.bz2$", full.names=TRUE)
        for(i in myfiles) R.utils::bunzip2(i, remove=TRUE)
    } else {
        message("To execute function, set 'rerun=TRUE'")
    }
}

## For making "lincs", "lincs_expr", "cmap", "cmap_expr", "cmap_rank" datasets
## from scratch, please refer to the vignette of this package by running
## `browseVignettes("signatureSearchData")` in R. 

################################
## make dtlink_db_clue_sti.db ##
################################

# This SQLite database contains drug-target links (dtlink) in DrugBank, CLUE, 
# and STITCH databases

## Get dtlink in DrugBank database (version 5.1.2)

### Download the `drugbank_all_target_uniprot_links.csv.zip` at 
### <https://www.drugbank.ca/releases/latest#external-links> under 
### `Target Drug-UniProt Links` section where `DRUG GROUP` is `All`.
### Extract the downloaded zip file, rename the extracted csv file as 
### `drug_target_uniprot_links_5.1.2.csv`, save it under `data` directory 

dt_uni_link <- read.delim("./data/drug_target_uniprot_links_5.1.2.csv",
                        sep=",", stringsAsFactors=FALSE)
#### Add gn_sym column for dt_uni_link
library(org.Hs.eg.db)
k <- keys(org.Hs.eg.db, keytype = "UNIPROT")
uni_gnsym <- suppressMessages(AnnotationDbi::select(
  org.Hs.eg.db, keys=k, columns=c("UNIPROT", "SYMBOL"), keytype="UNIPROT"))
dt_uni_link <- dplyr::as_tibble(dt_uni_link)
dt_uni_sym <- dplyr::left_join(dt_uni_link[,c("Name","UniProt.ID")], 
                               uni_gnsym, by = c("UniProt.ID"="UNIPROT"))
dtlink_db <- na.omit(unique(data.frame(drug_name=tolower(dt_uni_sym$Name), 
                        t_gn_sym=dt_uni_sym$SYMBOL, stringsAsFactors=FALSE)))
#### exclude drugs that have more than 100 targets in dtlink_db
dtlist_db <- split(dtlink_db$t_gn_sym, dtlink_db$drug_name)
dtnum_db <- sapply(dtlist_db, length)
sum(dtnum_db>100)
dtlist_db <- dtlist_db[dtnum_db<=100]
dtlink_db <- list2link(dtlist_db, type="dt")
#### exclude proteins that are targeted by more than 100 drugs in dtlink_db
tdlist_db <- split(dtlink_db$drug_name, dtlink_db$t_gn_sym)
tdnum_db <- sapply(tdlist_db, length)
sum(tdnum_db>100)
tdlist_db <- tdlist_db[tdnum_db<=100]
tdlink_db <- list2link(tdlist_db, type="td")
dtlink_db <- tdlink_db[,c("drug_name","t_gn_sym")]
length(unique(dtlink_db$drug_name)) # 5250
length(unique(dtlink_db$t_gn_sym)) # 2443
write.table(dtlink_db, "./data/dtlink_db_5.1.2.xls", 
            quote=FALSE, row.names=FALSE, sep="\t")

## Get dtlink in CLUE touchstone database at <https://clue.io/touchstone>
### Get perturbation information in CLUE touchstone (version 1.1.1.2) from API

#### Query the CLUE website at <https://clue.io/query> to get drugs in 
#### Touchstone database:
#### At Query window, select `Gene expression (L1000)` as query parameters. 
#### Select `individual query` as query mode. Then click the `example` link and
#### choose `PI3K/MTOR inhibitor` as an example query, the name of your query,
#### UP-regulated genes and DOWN-regulated genes would be automatically filled.
#### Then click `SUBMIT` button.
#### At the Query History window, select the query result, then click the 
#### `DETAILED LIST` button
#### At the result window, select `Compound x 2429` under `Perturbagen Type`.
#### Click `Export` button to save the result as `clue_example_mtor_res.txt` 
#### to your data directory under current R session. 

clue_res <- read.delim("./data/clue_example_mtor_res.txt",sep="\t") # 2429 X 6
clue_drugs <- as.character(unique(clue_res$Name)) # 2397
clue_pert_info_api <- getPertInfo(lincs_drugs)
saveRDS(clue_pert_info_api, "./data/clue_pert_info_api.rds")
clue_dt <- clue_pert_info_api[,c("pert_iname","target")]
clue_dtlist <- sapply(split(clue_dt$target, clue_dt$pert_iname), 
                       function(x) unique(unlist(x))) # 2397
dtlink_clue <- list2link(clue_dtlist, type="dt")
dtlink_clue$drug_name <- tolower(dtlink_clue$drug_name)
#### exclude drugs that have more than 100 targets, 
#### then exclude genes that have more than 100 targeted drugs
dtlist_clue <- split(dtlink_clue$t_gn_sym, dtlink_clue$drug_name)
dtnum_clue <- sapply(dtlist_clue, length)
sum(dtnum_clue > 100) # 0

tdlist_clue <- split(dtlink_clue$drug_name, dtlink_clue$t_gn_sym)
tdnum_clue <- sapply(tdlist_clue, length)
sum(tdnum_clue > 100)
tdlist_clue <- tdlist_clue[tdnum_clue <= 100]
tdlink_clue <- list2link(tdlist_clue, type="td")
dtlink_clue <- tdlink_clue[,c("drug_name","t_gn_sym")]
length(unique(dtlink_clue$drug_name)) # 1949
length(unique(dtlink_clue$t_gn_sym)) # 1325
write.table(dtlink_clue, "data/dtlink_clue_1.1.1.2.xls", 
            quote=FALSE, sep="\t", row.names=FALSE)

## Get dtlink in STITCH database (v5.0) 
### At STITCH Download page 
### <http://stitch.embl.de/cgi/download.pl?UserId=Z159fH9RYlUj&sessionId=2yGVo2kYC0ri>,
### Choose `Homo sapiens` organism, download and decompress the 
### `9606.protein_chemical.links.detailed.v5.0.tsv.gz` and
### `chemicals.v5.0.tsv.gz` as 
### `9606_protein_chemical_links_detailed_v5.0.tsv` and
### `chemicals.v5.0.tsv`, respectively, to the data directory 
### under your current working directory of R session. 

### Need to allocate large number of memory to read the table, 20Gb works
sti_dtlink_detail <- readr::read_tsv(
  "data/9606_protein_chemical_links_detailed_v5.0.tsv") # 15,473,939 X 7
sti_dtlink <- dplyr::filter(sti_dtlink_detail, 
                            experimental > 700 | database > 700) # 346,909 X 7
### Eliminate drugs that have more than 100 targets in `sti_dtlink`
sti_dtlist <- split(sti_dtlink$protein, sti_dtlink$chemical)
sti_dtnum <- sapply(sti_dtlist, length)
invalid_drugs <- names(sti_dtlist[sti_dtnum>100]) # 356
sti_dtlink <- dplyr::filter(sti_dtlink, ! chemical %in% invalid_drugs)
sti_dtlink$protein <- sub("9606.", "", sti_dtlink$protein) # 227,948 X 7
### get PubChem CID to drug name mappings from `chemicals.v5.0.tsv` file
sti_chem_info <- readr::read_tsv("data/chemicals.v5.0.tsv")
### Map Ensembl protein ids to gene symbol 
library(EnsDb.Hsapiens.v75)
edb <- EnsDb.Hsapiens.v75
gnames <- keys(edb, keytype = "GENENAME")
ensb_gn_protid <- na.omit(select(edb, keys = gnames, keytype = "GENENAME", 
                                 columns = c("GENENAME", "PROTEINID")))
sti_dtlink %<>% left_join(ensb_gn_protid, by = c("protein" = "PROTEINID")) %>% 
    left_join(sti_chem_info[,c("chemical","name")]) %>% 
    dplyr::select(drug_name=name, t_gn_sym=GENENAME) # 227,948 X 2
sti_dtlink <- unique(na.omit(sti_dtlink)) # 142291 X 2
sti_dtlink$drug_name <- sapply(sti_dtlink$drug_name, 
                               function(x) unlist(strsplit(x,", "))[1])
sti_dtlink$drug_name <- tolower(sti_dtlink$drug_name)
### Exclude proteins that are targeted by more than 100 drugs in sti_dtlink
#### check whether there are drugs that have more than 100 targets 
#### after ENSP id to gene SYMBOL mappings
dtlist_sti <- split(sti_dtlink$t_gn_sym, sti_dtlink$drug_name)
dtnum_sti <- sapply(dtlist_sti, length)
sum(dtnum_sti>100) # 0
tdlist_sti <- split(sti_dtlink$drug_name, sti_dtlink$t_gn_sym)
tdnum_sti <- sapply(tdlist_sti, length)
sum(tdnum_sti>100) # 255
tdlist_sti <- tdlist_sti[tdnum_sti<=100]
tdlink_sti <- list2link(tdlist_sti, type="td")
dtlink_sti <- tdlink_sti[,c("drug_name","t_gn_sym")] # 50,012 X 2
length(unique(dtlink_sti$drug_name)) # 18,847
length(unique(dtlink_sti$t_gn_sym)) # 5,119
write.table(dtlink_sti, "./data/dtlink_sti_v5.0.xls", 
            quote=FALSE, sep="\t", row.names=FALSE)

## Get dtlink tables in DrugBank/CLUE/STITCH databases and store in 
## SQLite database
### dtlink: drug_name-t_gn_sym and drug_name-ENTREZ 
### (Note: drug_name in dtlink are all lowercase)
#### Combine dtlink in DrugBank/CLUE/STITCH databases
dtlink_db <- read.delim("data/dtlink_db_5.1.2.xls", stringsAsFactors=FALSE)
dtlink_clue<-read.delim("data/dtlink_clue_1.1.1.2.xls", stringsAsFactors=FALSE)
dtlink_sti <- read.delim("data/dtlink_sti_v5.0.xls", stringsAsFactors=FALSE)
dtlink <- rbind(dtlink_db, dtlink_clue, dtlink_sti) %>% unique
#### Exclude proteins or drugs that have more than 100 targets
dtlist <- split(dtlink$t_gn_sym, dtlink$drug_name)
tdlist <- split(dtlink$drug_name, dtlink$t_gn_sym)
dt_num <- sapply(dtlist, length)
sum(dt_num>100) # 0
td_num <- sapply(tdlist, length)
sum(td_num>100) # 29
tdlist <- tdlist[td_num<=100]
tdlink <- list2link(tdlist, type="td")
dtlink <- tdlink[,c("drug_name","t_gn_sym")] # 62,228 X 2
write.table(dtlink, "data/dtlink3.xls", quote=FALSE, row.names=FALSE, sep="\t")
length(unique(dtlink$drug_name)) # 22647
length(unique(dtlink$t_gn_sym)) # 5729
#### Map gene SYMBOL to entrez id to get drug_name-entrez links
library(org.Hs.eg.db)
sym <- AnnotationDbi::keys(org.Hs.eg.db, keytype = "SYMBOL")
sym_entrez <- AnnotationDbi::select(org.Hs.eg.db, keys = sym, 
                                    keytype = "SYMBOL", columns = "ENTREZID")
dtlink_entrez <- left_join(as_tibble(dtlink), sym_entrez, 
                           by = c("t_gn_sym"="SYMBOL")) %>% 
                 dplyr::select(drug_name, ENTREZID) %>% 
                 distinct(.keep_all = TRUE)
write.table(dtlink_entrez, "data/dtlink_entrez3.xls", quote=FALSE, 
            row.names=FALSE, sep="\t")
### Store dtlinks in SQLite database
library(RSQLite)
mydb <- dbConnect(SQLite(), "data/dtlink_db_clue_sti.db")
dbWriteTable(mydb, "dtlink_db", dtlink_db, overwrite = TRUE)
dbWriteTable(mydb, "dtlink_clue", dtlink_clue, overwrite = TRUE)
dbWriteTable(mydb, "dtlink_sti", dtlink_sti, overwrite = TRUE)
dbWriteTable(mydb, "dtlink", dtlink, overwrite = TRUE)
dbWriteTable(mydb, "dtlink_entrez", dtlink_entrez, overwrite = TRUE)
dbListTables(mydb)
dbDisconnect(mydb)

#####################
## make goAnno.rds ##
#####################

# It is an intermediate file storing the GO to GENE SYMBOLE mapping information
# for `dtnetplot` function in `signatureSearch` package

OrgDb <- GOSemSim::load_OrgDb(OrgDb = "org.Hs.eg.db")
kk <- keys(OrgDb, keytype="SYMBOL")
goAnno <- AnnotationDbi::select(OrgDb, keys=kk, keytype="SYMBOL",
                                columns=c("GOALL", "ONTOLOGYALL"))
goAnno <- unique(na.omit(goAnno)) # 3,430,396 X 4
saveRDS(goAnno, "./data/goAnno.rds")

##########################
## make goAnno_drug.rds ##
##########################

# It is an intermediate file storing the GO to drug name mapping information
# for `dsea` functions in `signatureSearch` package

## convert GOALL-SYMBOL in goAnno to GOALL-drug
### Get drug-target mapping information in DrugBank, CLUE and STITCH databases 
### from `dtlink_db_clue_sti.db` created above
library(RSQLite); library(dplyr)
conn <- dbConnect(SQLite(), "./data/dtlink_db_clue_sti.db")
dtlink <- dbGetQuery(conn, 'SELECT * FROM dtlink') # 62,228 X 2
dbDisconnect(conn)
### Convert GO-GENE mappings to GO-drug mappings
goAnno <- readRDS("./data/goAnno.rds")
goAnno_drug <- left_join(as_tibble(goAnno[,c(2,1,4)]), dtlink, 
                         by = c("SYMBOL"="t_gn_sym"))
goAnno_drug <- as.data.frame(goAnno_drug)[,c("GOALL","ONTOLOGYALL","drug_name")]
goAnno_drug <- na.omit(goAnno_drug)
goAnno_drug <- goAnno_drug[!duplicated(goAnno_drug[,c("GOALL","drug_name")]),]
saveRDS(goAnno_drug, "./data/goAnno_drug.rds") # 7,517,403 X 3

######################
## make GO_DATA.rds ##
######################

# It is an intermediate file storing the GO annotation environment (GO term to 
# gene sets, gene to GO terms, GO term ID to description, GO ID to ontology) 
# used for `tsea` methods to accelerate the speed by avoiding building this 
# environment every time running the tsea functions 

GO_DATA <- clusterProfiler:::get_GO_data(
  OrgDb="org.Hs.eg.db", ont="ALL", keytype="SYMBOL")
saveRDS(GO_DATA, "./data/GO_DATA.rds")

###########################
## make GO_DATA_drug.rds ##
###########################

# It is an intermediate file storing the GO annotation environment (GO term to 
# drug sets, drug to GO terms, GO term ID to description, GO ID to ontology) 
# used for `dsea` methods to accelerate the speed by avoiding building this 
# environment every time running the dsea functions 

## Read in `goAnno_drug.rds` generated above
goAnno_drug <- readRDS("data/goAnno_drug.rds")
GO2GENE <- goAnno_drug[,c("GOALL","drug_name")]
GO_DATA_drug <- clusterProfiler:::build_Anno(GO2GENE, 
                                  clusterProfiler:::get_GO2TERM_table())
saveRDS(GO_DATA_drug, "./data/GO_DATA_drug.rds")

#########################
## make taurefList.rds ##
#########################

# It uses all signatures in the reference database (such as LINCS) to query 
# against itself as Qref to compute tau score of `gess_lincs` 
# method in `signatureSearch` package. Tau score compares observed enrichment 
# score to all others in Qref. It represents the percentage of reference queries
# with a lower |NCS| than |NCSq,r|, adjusted to retain the sign of NCSq,r. 
# NCSq,r is the normalized connectivity score for signature r relative to 
# query q. A tau of 90 indicates that only 10 percent of reference perturbations
# showed stronger connectivity to the query. For more details, please refer to 
# Subramanian et al., 2017, Cell, A Next Generation Connectivity Map: L1000 
# Platform and the First 1,000,000 Profiles

## Create Query Reference DB for Tau Score Computation of `gess_lincs` method
### Load `lincs` database created above
library(HDF5Array); library(SummarizedExperiment)
se = loadHDF5SummarizedExperiment("./data/lincs")
score_mat = assay(se)
### Create query list for all signatures in se
query_list <- lapply(colnames(score_mat), function(x) {
  vec <- as.matrix(score_mat[,x])
  names(vec) <- rownames(score_mat)
  sigvec = sort(vec, decreasing = TRUE)
  list(upset=utils::head(names(sigvec), 150), 
       downset=utils::tail(names(sigvec), 150))
})
names(query_list) = colData(se)$pert_cell_factor
###  Query signatures in the query_list against lincs database
###  To save time, the processing is paralleled with BiocParallel to run on 
###  CPU cores of a computer cluster with a scheduler (e.g. Slurm). 
#### Define submission function
f <- function(x, se, query_list, dest_dir) {
  require(signatureSearch)
  chunkno <- x 
  sz <- 10 # small enough to use short queue 
  qlist <- split(query_list, ceiling(seq_along(names(query_list))/sz))
  myMA <- matrix(NA, length(query_list), sz, 
                 dimnames=list(names(query_list), seq_len(sz)))
  qlistone <- qlist[[chunkno]] 
  for(i in seq_along(qlistone)){
    qsig <- suppressMessages(qSig(qsig=list(upset=qlistone[[i]]$up, 
                                            downset=qlistone[[i]]$down), 
                 gess_method = "LINCS", refdb = se))
    lincs <- signatureSearch::gess_lincs(qsig, sortby=NA)
    resultDF <- result(lincs)
    ncs <- resultDF$NCS
    mynames <- paste(resultDF$pert, resultDF$cell, resultDF$type, sep="__")
    names(ncs) <- mynames
    myMA[,i] <- ncs[rownames(myMA)]
    colnames(myMA)[i] <- names(qlistone[i])
  }
  myMA <- myMA[, seq_along(qlistone)] 
  ## Only relevant for last entry that may not have as many columns as sz
  if(! dir.exists(dest_dir)) dir.create(dest_dir)
  write.table(as.data.frame(myMA), 
              file=paste0(dest_dir, "/result_", sprintf("%03d", chunkno)), 
              col.names=NA, quote=FALSE, sep="\t")
}
#### Split query into small chunks, each chunk will be processed on computer 
#### cluster as one process. Use `batchtools` to manage job submission.
#### Please make sure that `data` directory exists under your current working
#### directory of R session
library(batchtools)
sz <- 10 # small enough to finish each chunk within 2 hours
qlist <- split(query_list, ceiling(seq_along(names(query_list))/sz))
if(! file.exists("slurm.tmpl")) 
  download.file("https://goo.gl/tLMddb", "slurm.tmpl", quiet=TRUE)
if(! file.exists(".batchtools.conf.R")) 
  download.file("https://goo.gl/5HrYkE", ".batchtools.conf.R", quiet=TRUE)
if(dir.exists("data/tau_queries_reg")) 
  unlink("data/tau_queries_reg", recursive=TRUE)
reg = makeRegistry(file.dir="data/tau_queries_reg", 
                   conf.file=".batchtools.conf.R")
dest_dir="data/tau_queries" 
## directory to store the intermediate results generated by function f
ids = batchMap(f, x = seq(along=qlist), 
            more.args = list(se=se, query_list=query_list, dest_dir=dest_dir))
#testJob(id = 1)
done <- submitJobs(ids, resources=list(walltime=36000, ncpus=1, memory=10240), 
                   partition="short", reg=reg)
#waitForJobs()
getStatus()

#### Inspect result and resubmit jobs for missing and empty files
fileDF <- file.info(list.files(dest_dir, pattern="result_*", full.names=TRUE))
index_empty <- as.numeric(gsub(".*_", "", row.names(fileDF[fileDF$size==0, ])))
qlist <- split(query_list, ceiling(seq_along(names(query_list))/sz))
index_all_files <- seq_along(qlist)
index_exist <- as.numeric(gsub(".*_", "", row.names(fileDF)))
index_missing <- index_all_files[!index_all_files %in% index_exist]
index_repeat <- unique(sort(c(index_empty, index_missing)))
if(length(index_repeat)!=0){
  if(dir.exists("data/tau_queries_reg")) 
    unlink("data/tau_queries_reg", recursive=TRUE)
  reg = makeRegistry(file.dir="data/tau_queries_reg", 
                     conf.file=".batchtools.conf.R")
  ids = batchMap(f, x = index_repeat, more.args = 
                   list(se=se, query_list=query_list, dest_dir=dest_dir))
  done <- submitJobs(ids, resources=list(walltime=36000, ncpus=1, 
                                    memory=10240, partition="short"), reg=reg)
}
## Repeat the above steps until all jobs are successfully completed 
## (index_repeat is empty)

#### Organize results in list where each component contains data.frame with 
#### query results from one cell type
pathDF <- data.frame(query=names(query_list), 
                     path=rep(paste0("data/tau_queries/result_", 
                                     sprintf("%03d", seq_along(qlist))), 
                              sapply(qlist, length)))
pathDF <- data.frame(pathDF, target=gsub("^.*?__", "", pathDF$query))
pathList <- split(as.character(pathDF$path), factor(pathDF$target))
pathList <- sapply(pathList, unique) # eliminates duplicated imports of files
taurefList <- rep(NA, length(pathList)); names(taurefList) <- names(pathList)
taurefList <- as.list(taurefList) 
for(celltype in names(pathList)) {
  for(i in seq_along(pathList[[celltype]])) {
    tmpDF <- read.delim(pathList[[celltype]][i], row.names=1, check.names=FALSE)
    tmpDF <- round(tmpDF, 2) # Reduces data size
    colindex <- gsub("^.*?__", "", colnames(tmpDF)) %in% celltype
    tmpDF <- tmpDF[, colindex, drop=FALSE] 
    if(i==1) { 
      rowindex <- gsub("^.*?__", "", rownames(tmpDF)) %in% celltype
      containerDF <- tmpDF[rowindex, , drop=FALSE]
    } else {
      containerDF <- cbind(containerDF, tmpDF[rownames(containerDF),])
    }
    print(paste("Finished", i, "of", length(pathList[[celltype]]), 
                "results collected in", ncol(containerDF), "columns."))
  }
  taurefList[[celltype]] <- containerDF
  rm(containerDF); gc()
}
saveRDS(taurefList, file="data/tau_queries/taurefList.rds")

######################
## make ES_NULL.txt ##
######################

#  ES null distribution is generated with random queries for computing nominal 
#  P-values for Enrichment Score in gess_lincs result. 
#  To save time, the codes below parallel the process
#  with BiocParallel to run on CPU cores of a computer cluster with Slurm 
#  scheduler. It uses 1000 random queries searching against the lincs database
#  to generate the null distribution of ES.

## Create list of random queries
se = loadHDF5SummarizedExperiment("./data/lincs")
idnames <- rownames(se)
query_list <- signatureSearch:::randQuerySets(
  id_names=idnames, N_queries=1000, set_length=150)
## Define submission function
f <- function(x, query_list, se, dest_dir) {
  require(signatureSearch)
  chunkno <- x 
  sz <- 10 # small enough to run within 2 hours 
  qlist <- split(query_list, ceiling(seq_along(names(query_list))/sz)) 
  qlistone <- qlist[[chunkno]] 
  for(i in seq_along(qlistone)){
    esout <- signatureSearch:::lincsEnrich(
      se, upset=qlistone[[i]]$up, downset=qlistone[[i]]$down, 
      sortby=NA, output="esonly", type=1)
    names(esout) <- colnames(se)
    wtcs = esout
    if(i==1) 
      myMA <- matrix(NA, length(esout), length(qlistone), 
                     dimnames=list(names(wtcs), seq_along(qlistone)))
    myMA[,i] <- wtcs[rownames(myMA)]
    colnames(myMA)[i] <- names(qlistone[i])
  }
  write.table(as.data.frame(myMA), 
              file=paste0(dest_dir, "/result_", sprintf("%03d", chunkno)), 
              col.names = NA, quote=FALSE, sep="\t")
}
## Split query into chunks, each chunk will be processed on cluster as 
## one process
sz <- 10 # small enough to finish each chunk within 2 hours
qlist <- split(query_list, ceiling(seq_along(names(query_list))/sz)) 
dest_dir="data/es_queries" 
if(! file.exists("slurm.tmpl")) 
  download.file("https://goo.gl/tLMddb", "slurm.tmpl", quiet=TRUE)
if(! file.exists(".batchtools.conf.R")) 
  download.file("https://goo.gl/5HrYkE", ".batchtools.conf.R", quiet=TRUE)
if(dir.exists("data/es_queries_reg")) 
  unlink("data/es_queries_reg", recursive=TRUE)
reg = makeRegistry(file.dir="data/es_queries_reg", 
                   conf.file=".batchtools.conf.R")
ids = batchMap(f, x = seq(along=qlist), more.args = 
                 list(query_list=query_list, se=se, dest_dir=dest_dir))
#testJob(id = 1)
done <- submitJobs(ids, resources=list(walltime=36000, ncpus=1, memory=10240, 
                                       partition="short"), reg=reg)
getStatus()

## Inspect result and resubmit jobs for missing and empty files
fileDF <- file.info(list.files(dest_dir, pattern="result_*", full.names=TRUE))
index_empty <- as.numeric(gsub(".*_", "", row.names(fileDF[fileDF$size==0, ])))
qlist <- split(query_list, ceiling(seq_along(names(query_list))/sz))
index_all_files <- seq_along(qlist)
index_exist <- as.numeric(gsub(".*_", "", row.names(fileDF)))
index_missing <- index_all_files[!index_all_files %in% index_exist]
index_repeat <- unique(sort(c(index_empty, index_missing)))
if(length(index_repeat)!=0){
  if(dir.exists("data/es_queries_reg")) 
    unlink("data/es_queries_reg", recursive=TRUE)
  reg = makeRegistry(file.dir="data/es_queries_reg", 
                     conf.file=".batchtools.conf.R")
  ids = batchMap(f, x = index_repeat, more.args = 
                   list(se=se, query_list=query_list, dest_dir=dest_dir))
  done <- submitJobs(ids, resources=list(walltime=36000, ncpus=1, memory=10240,
                                         partition="short"), reg=reg)
}
## Repeat the above steps until all jobs are successfully completed 
## (index_repeat is empty)
## 
## If all jobs are successfully completed, collect results in frequency table 
## with 3 digit accuracy
esMA <- data.frame(WTCS=as.character(round(rev(seq(-1, 1, by=0.001)), 3)), 
                   Freq=0, stringsAsFactors=FALSE) 
myfiles <- list.files(dest_dir, "result_\\d{3,3}", full.names=TRUE)
for(i in myfiles) {
  df <- read.delim(i, row.names=1)
  freq <- table(round(as.numeric(as.matrix(df),3),3)) 
  # processes entire data.frame
  freq <- freq[as.character(esMA[,1])]
  freq[is.na(freq)] <- 0
  esMA[,"Freq"] <- as.numeric(esMA[,"Freq"]) + as.numeric(freq)
  print(paste("Processed", i))
}
write.table(esMA, file=paste0(dest_dir,"/ES_NULL.txt"), quote=FALSE, row.names=FALSE, sep="\t")

#' This function generates a list of treatment samples and control samples CEL ids
#' for each compound treatment from the experiment annotation of CMAP02 
#' instances. For more context of the CMAP02 database, please consult the vignette
#' of this package. 
#' @title Generate List of Treatment vs. Control Samples 
#' @param cmap data.frame containing experiment information of CMAP02 instances
#' @param myby "CMP" or "CMP_CELL", "CMP": by compound treatments in all cells;
#' "CMP_CELL": by compound treatments in individual cell
#' @return a list object containing treatment samples and control samples ids
#' for each compound treament. 
#' @examples 
#' path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
#' cmap_inst <- read.delim(path, check.names=FALSE) 
#' comp_list <- sampleList(cmap_inst, myby="CMP_CELL")
#' @export
sampleList <- function(cmap, myby) {
    ## Reconstruct CEL file names for control samples
    byCel_list <- split(cmap[, c("perturbation_scan_id", "vehicle_scan_id4")], 
                        as.character(cmap$perturbation_scan_id))
    sampleList1 <- function(x) {
        c <- unlist(strsplit(as.character(x[,2]), "\\."))
        if(max(nchar(c)>4)) {    
            c <- gsub("'", "", as.character(x[,2]))
        } else {
            c <- c[nchar(c)!=0]
            c <- paste0(gsub("\\..*", "", as.character(x[,1])), ".", c)
            c <- gsub("'", "", c) # Quite a few CEL file names have a "'" prepended 
        }
        t <- gsub("'", "", as.character(x[,1]))
        return(list(list(t=t, c=c)))
    }
    byCel_list <- sapply(byCel_list, sampleList1)
    
    ## Split by CMP only, or CMP and cell type
    if(myby=="CMP") {
        byCMP_list <- split(cmap[, c("cmap_name", "perturbation_scan_id")], 
                            as.character(cmap$cmap_name))
    }
    if(myby=="CMP_CELL") {
        cmap <- data.frame(cmp_cell=paste(cmap$cmap_name, cmap$cell2, sep="_"), cmap)
        byCMP_list <- split(cmap[, c("cmap_name", "perturbation_scan_id")], 
                            as.character(cmap$cmp_cell))
    }
    sampleList2 <- function(x, y=byCel_list) {
        s <- y[x[,2]]    
        l <- list(t=paste0(unique(as.character(unlist(sapply(seq_along(s), function(x) s[[x]]$t)))), ".CEL"),
                  c=paste0(unique(as.character(unlist(sapply(seq_along(s), function(x) s[[x]]$c)))), ".CEL"))
        return(list(l))
    }
    byCel_list <- sapply(names(byCMP_list), function(i) sampleList2(x=byCMP_list[[i]], y=byCel_list))
    return(byCel_list)
}

meta <- data.frame(
  Title = c("cmap", "cmap_expr", "cmap_rank",
            "lincs", "lincs_expr",
            "dtlink_db_clue_sti",
            "goAnno", "goAnno_drug", 
            "GO_DATA", "GO_DATA_drug", 
            "taurefList", "ES_NULL",
            "lincs2"),
  Description = c("CMAP log2FC matrix stored in h5 file",
                  "CMAP expression matrix stored in h5 file",
                  "CMAP rank matrix stored in h5 file",
                  "LINCS mod-z scores matrix stored in h5 file",
                  "LINCS expression matrix stored in h5 file",
                  "SQLite database containing drug-target links in drugbank, LINCS and STITCH databases",
                  "GO terms to gene symbols annotation",
                  "GO to drugs annotation after drug to gene/target mapping",
                  "GO annotation environment",
                  "GO annotation environment after drug mappings",
                  "Reference list for computing tau scores of gess_lincs method",
                  "Null distribution of Enrichment Score for computing nominal P-values",
                  "LINCS beta 2020 level 5 mod-z score matrix stored in h5 file"
                  ),
  BiocVersion = c(rep("3.9", 12), "3.14"),
  Genome = rep("GRCh38", 13),
  SourceType = c("CEL", "CEL", "TXT",
                 "HDF5", "HDF5",
                 "CSV",
                 "MySQL", "MySQL",
                 "MySQL", "MySQL", 
                 "RData", "RData",
                 "HDF5"),
  SourceUrl = c("https://portals.broadinstitute.org/cmap/cel_file_chunks.jsp",
                "https://portals.broadinstitute.org/cmap/cel_file_chunks.jsp",
                "ftp://ftp.broad.mit.edu/pub/cmap/rankMatrix.txt.zip",
                "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE92742&format=file&file=GSE92742%5FBroad%5FLINCS%5FLevel5%5FCOMPZ%2EMODZ%5Fn473647x12328%2Egctx%2Egz",
                "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE92742&format=file&file=GSE92742%5FBroad%5FLINCS%5FLevel3%5FINF%5Fmlr12k%5Fn1319138x12328%2Egctx%2Egz",
                "https://www.drugbank.ca/, https://clue.io/, http://stitch.embl.de/",
                "https://bioconductor.org/packages/release/data/annotation/html/GO.db.html",
                "https://bioconductor.org/packages/release/data/annotation/html/GO.db.html",
                "https://bioconductor.org/packages/release/data/annotation/html/GO.db.html",
                "https://bioconductor.org/packages/release/data/annotation/html/GO.db.html",
                "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742",
                "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742",
                "https://s3.amazonaws.com/macchiato.clue.io/builds/LINCS2020/level5/level5_beta_trt_cp_n720216x12328.gctx"
                ),
  SourceVersion = c("build02", "build02", "build02",
                    "Mar 03 2017", "Mar 03 2017",
                    "5.0.10; 1.1.1.2; v5.0",
                    "3.7.0",
                    "3.7.0",
                    "3.7.0",
                    "3.7.0",
                    "Mar 03 2017",
                    "Mar 03 2017",
                    "beta"),
  Species = rep("Homo sapiens", 13),
  TaxonomyId = rep(9606, 13),
  Coordinate_1_based = rep(TRUE, 13),
  DataProvider = c("Broad Institute", "Broad Institute", "Broad Institute",
                   "Broad Institute", "Broad Institute",
                   "DrugBank, Broad Institute, STITCH",
                   "GO", "GO", "GO", "GO", 
                   "Broad Institute", "Broad Institute",
                   "Broad Institute"),
  Maintainer = c(rep("Yuzhu Duan <yduan004@ucr.edu>", 12), "Brendan Gongol <bgong001@ucr.edu>"),
  RDataClass = c("character", "character", "character",
                 "character", "character",
                 "character", "data.frame", "data.frame",
                 "environment", "environment", "list", "data.frame",
                 "character"),
  DispatchClass = c("FilePath", "FilePath", "FilePath",
                    "FilePath", "FilePath",
                    "FilePath",
                    "RDS", "RDS", "RDS", "RDS", "RDS", "EpiMetadata",
                    "FilePath"),
  RDataPath = c("signatureSearchData/v0.1/cmap.h5",
                "signatureSearchData/v0.1/cmap_expr.h5",
                "signatureSearchData/v0.1/cmap_rank.h5",
                "signatureSearchData/v0.1/lincs.h5",
                "signatureSearchData/v0.1/lincs_expr.h5",
                "signatureSearchData/v0.1/dtlink_db_clue_sti.db", 
                "signatureSearchData/v0.1/goAnno.rds", 
                "signatureSearchData/v0.1/goAnno_drug.rds",
                "signatureSearchData/v0.1/GO_DATA.rds", 
                "signatureSearchData/v0.1/GO_DATA_drug.rds",
                "signatureSearchData/v0.1/taurefList.rds",
                "signatureSearchData/v0.1/ES_NULL.txt",
                "signatureSearchData/v0.2/lincs2020.h5"),
  Tags = c("CMAP", "CMAP_expr", "CMAP_rank",
           "LINCS", "LINCS_expr",
           "drug target", "GO", "GO",
           "GO","GO","lincs_tau","lincs_esnull",
           "LINCS2")
)
write.csv(meta, file="inst/extdata/metadata.csv", row.names=FALSE)

#' Function combines batch-processed normalization results from same chip type
#' into a single data frame. Typically, the input for this step is generated by
#' the upstream \code{normalizeCel} function. For more context, please consult
#' the vignette of this package.
#' @title Combine Batch-processed Normalization Results
#' @param chiptype_dir character vector containing paths to chiptype directories
#' @param rerun TRUE or FALSE, whether to run the function
#' @return File storing normalization values for each CEL file type
#' @importFrom utils read.delim
#' @importFrom utils write.table
#' @examples 
#' # chiptype_dir <- unique(readRDS("./data/chiptype.rds"))
#' combineResults(chiptype_dir, rerun=FALSE)
#' @export
combineResults <- function(chiptype_dir, rerun=TRUE) {
    if(rerun) {
        for(j in seq_along(chiptype_dir)) {
            mydirs <- list.files(paste0("data/", chiptype_dir[j]), 
                                 pattern="cellbatch_", full.names=TRUE)
            for(i in seq_along(mydirs)) {
                if(i==1) {
                    df1 <- read.delim(paste0(mydirs[i], "/", "mas5exprs.xls"), 
                                      row.names=1, check.names=FALSE)
                } else {
                    tmpdf1 <- read.delim(paste0(mydirs[i], "/", "mas5exprs.xls"), 
                                         row.names=1, check.names=FALSE)
                    df1 <-cbind(df1, tmpdf1)
                }
                cat("Processed", i, "of", length(mydirs), "\n")
            }
            write.table(df1, paste0("data/", chiptype_dir[j], "/all_mas5exprs.xls"), 
                        quote=FALSE, sep="\t", col.names = NA)
            # For fast loading
            saveRDS(df1, paste0("data/", chiptype_dir[j], "/", "all_mas5exprs.rds")) 
            cat("Generated", paste0("data/", chiptype_dir[j], "/all_mas5exprs.xls"), "\n")
        }
    } else {
        print("To execute function, set 'rerun=TRUE'")
    }
}

#' Function assembles normalization results from different chip types
#' into a single data frame. Note, this is possible since the three chip types
#' used by the CMap2 project contain nearly identical probe set ids. Typically,
#' the input for this step is generated by the upstream \code{normalizeCel} and
#' \code{combineResults} functions. For more context, please consult the vignette
#' of this package.
#' @title Combine Normalized Data from Different Chip Types
#' @param chiptype_dir character vector containing paths to chiptype directories
#' @param norm_method normalization method, one of "MAS5" or "rma".
#' @return Object of class \code{data.frame} containing normalization values
#' @examples 
#' # chiptype_dir <- unique(readRDS("./data/chiptype.rds"))
#' combineNormRes(chiptype_dir, norm_method="not run")
#' @export
combineNormRes <- function(chiptype_dir, norm_method) {
    if(norm_method=="MAS5"){
        df1 <- readRDS(paste0("data/", chiptype_dir[1], "/", "all_mas5exprs.rds"))
        df2 <- readRDS(paste0("data/", chiptype_dir[2], "/", "all_mas5exprs.rds"))
        df3 <- readRDS(paste0("data/", chiptype_dir[3], "/", "all_mas5exprs.rds"))
    } else if (norm_method=="rma"){
        df1 <- read.delim(paste0("./data/", chiptype_dir[1], "/rma_exprs.xls"), 
                          sep="\t", header=TRUE, row.names=1, check.names=FALSE)
        df2 <- read.delim(paste0("./data/", chiptype_dir[2], "/rma_exprs.xls"), 
                          sep="\t", header=TRUE, row.names=1, check.names=FALSE)
        df3 <- read.delim(paste0("./data/", chiptype_dir[3], "/rma_exprs.xls"), 
                          sep="\t", header=TRUE, row.names=1, check.names=FALSE)
    } else {
        message("Please set norm_method as one of 'MAS5' or 'rma'!")
        return(NULL)
    }
    affyid <- rownames(df1)[rownames(df1) %in% rownames(df2)]
    affyid <- affyid[affyid %in% rownames(df3)]
    normdf <- cbind(df1[affyid,], df2[affyid,], df3[affyid,])
    return(normdf)
}

#' Transform Probe Set to Gene Level Data
#' 
#' Function to transform expression values from probe set to gene level data. If
#' genes are represented by several probe sets then their mean intensities are used.
#' Probe sets not matching any gene ids are removed.
#' @param expr_df data.frame with probe set level expression data
#' @param annot data.frame containing probe set id (in row name slot) to
#' gene entrez id (in first column) mapping information
#' @return data.frame with gene level expression values
#' @examples
#' expr_df <- data.frame(t1=runif(3), t2=runif(3), row.names=paste0("p", 1:3))
#' annot <- data.frame(ENTREZID=c("123","123","124"), 
#'                     SYMBOL=c("g1","g1","g2"),
#'                     row.names=paste0("p", 1:3))
#' gdf <- probe2gene(expr_df, annot)
#' @export
probe2gene <- function(expr_df, annot){
    myAnnot <- annot[as.character(annot[,"ENTREZID"]) != "NA",]
    expr_df <- expr_df[rownames(myAnnot),]
    idlist <- tapply(row.names(myAnnot), as.character(myAnnot$ENTREZID), c)
    gexpr_df <- t(sapply(names(idlist), function(x) colMeans(expr_df[idlist[[x]], ])))
    return(gexpr_df)
}

#' Calculate Mean Value for Replicated Samples
#' 
#' Function averages the normalized gene expression values for different
#' concentrations, treatment times and replicates of compounds and cell types.
#' For more context on this step, please consult the corresponding section in the
#' vignette of this package.
#' @param expr_df \code{data.frame} containing normalized expression values
#' @param cmap_inst \code{data.frame} defining experimental conditions
#' @return \code{data.frame} with mean expression values of replicates
#' @examples
#' path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
#' cmap_inst <- read.delim(path, check.names=FALSE) 
#' expr_df <- as.data.frame(matrix(runif(30), ncol=5))
#' colnames(expr_df) <- paste0(cmap_inst$perturbation_scan_id[seq_len(5)],".CEL")
#' mexpr <- meanExpr(expr_df, cmap_inst[seq_len(5),])
#' @export
meanExpr <- function(expr_df, cmap_inst){
    cmap_inst <- data.frame(cmap_inst, 
            drug_cell = paste(cmap_inst$cmap_name, cmap_inst$cell2, sep ="_"))
    celid <- paste0(cmap_inst$perturbation_scan_id,".CEL")
    celid2 <- gsub("'", "", celid) # get rid of "'" in celid
    drug_celid_list <- tapply(celid2, cmap_inst$drug_cell, c) 
    drug_cell_expr <- sapply(names(drug_celid_list), function(x) 
        rowMeans(as.data.frame(expr_df[ ,drug_celid_list[[x]]])))
    return(drug_cell_expr)
}

#' This function runs DEG analysis with Limma at user defined FDR and LFC
#' cutoff by providing CMAP02 normalized expression values and annotation of
#' treatment v.s. control samples. For more context of the CMAP02 database, 
#' please consult the vignette of this package. 
#' @title DEG Analysis with Limma
#' @param df data.frame containing normalized intensity values of CMAP02 samples
#' @param comp_list list of CEL file ids of treatment and control samples for 
#' each compound treatment. The list for CMAP02 data is generated from 
#' the \code{\link{sampleList}} function.
#' @param fdr cutoff of false discovery rate (FDR) for defining DEGs
#' @param foldchange cutoff of log2 fold change (LFC) for defining DEGs
#' @param verbose TRUE or FALSE
#' @return list containing DEGs and log2FC matrix
#' @importFrom Biobase ExpressionSet
#' @importFrom limma lmFit
#' @importFrom limma makeContrasts
#' @importFrom limma contrasts.fit
#' @importFrom limma eBayes
#' @importFrom limma topTable
#' @importFrom stats model.matrix
#' @examples 
#' path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
#' cmap_inst <- read.delim(path, check.names=FALSE) 
#' comp_list <- sampleList(cmap_inst, myby="CMP_CELL")
#' df <- as.data.frame(matrix(runif(70), ncol=7))
#' colnames(df) <- unlist(comp_list[1])
#' degList <- runLimma(df, comp_list[1])  
#' @export
runLimma <- function(df, comp_list, fdr=0.05, foldchange=1, verbose=TRUE) {
    ## Generate result container
    deg <- matrix(0, nrow=nrow(df), ncol=length(comp_list), 
                  dimnames = list(rownames(df), names(comp_list)))
    colnames(deg) <- names(comp_list)
    #deg_list <- NULL # only used if affyid not NULL
    logfc_ma <- deg
    pvalue_ma <- deg
    ## Run limma
    for(i in seq_along(comp_list)) {
        sample_set <- unlist(comp_list[[i]])
        repcounts <- sapply(comp_list[[i]], length)
        repcounts <- paste("rep count:", paste(paste0(names(repcounts), repcounts), 
                                               collapse="_"))
        ## The following if statement will skip sample sets with less than 3 CEL files 
        ## (control and treatment) or those containing non-existing CEL files. 
        ## For tracking NAs will be injected in the corresponding columns of the
        ## result matrix.
        if((length(sample_set)<3) | any(!sample_set %in% colnames(df))) {
            deg[, i] <- NA
            if(verbose==TRUE) {
                cat("Sample", i, "of", paste0(length(comp_list), ":"), 
                    paste0("not enough usable replicates (", repcounts, ")."), "\n")
            }
        } else {
            dfsub <- df[, sample_set]
            # eset <- new("ExpressionSet", exprs = as.matrix(dfsub), annotation="hgu133a")
            eset <- Biobase::ExpressionSet(assayData=as.matrix(dfsub),  annotation="hgu133a")
            repno <- rep(seq_along(comp_list[[i]]), sapply(comp_list[[i]], length))
            design <- model.matrix(~ -1+factor(repno))
            colnames(design) <- names(comp_list[[i]])
            # Fit a linear model for each gene based on the given series of arrays
            fit <- limma::lmFit(eset, design) 
            contrast.matrix <- limma::makeContrasts(contrasts="t-c", levels=design)
            fit2 <- limma::contrasts.fit(fit, contrast.matrix)
            # Computes moderated t-statistics and log-odds of differential expression 
            # by empirical Bayes shrinkage of the standard errors towards a common value.
            fit2 <- limma::eBayes(fit2)
            limmaDF <- limma::topTable(fit2, coef=1, adjust="fdr", sort.by="none", number=Inf)
            # if(!is.null(affyid)) {
            #     tmp_list <- list(limmaDF[affyid,])
            #     names(tmp_list) <- names(comp_list[i])
            #     deg_list <- c(deg_list, tmp_list)
            # } 
            pval <- limmaDF$adj.P.Val <= fdr # FDR 1%
            # Fold change 2
            fold <- (limmaDF$logFC >= foldchange | limmaDF$logFC <= -foldchange) 
            affyids <- rownames(limmaDF[pval & fold,])
            deg[affyids, i] <- 1
            pvalue_ma[ , i] <- limmaDF[, "adj.P.Val"]
            logfc_ma[ , i] <- limmaDF[,"logFC"]
            if(verbose==TRUE) {
                cat("Sample", i, "of", paste0(length(comp_list), ":"), "identified", 
                    length(affyids), paste0("DEGs (", repcounts, ")."), "\n")
            }
        }
    }
    # if(!is.null(affyid)) {
    #     return(deg_list)
    #} 
    col_index <- !is.na(colSums(deg)) # Remove columns where DEG analysis was not possible
    return(list(DEG=deg[, col_index], logFC=logfc_ma[, col_index], FDR=pvalue_ma[, col_index]))
}

#' Function processes the CEL files from each chip type (three for CMap2 data)
#' separately using the MAS5 normalization algorithm. The results will be written
#' to subdirectores (under a data parent directory) that are named after the chip
#' type names. To reduce the memory consumption of this step, the CEL files are
#' processed in user definable batch sizes. For details on the overall workflow,
#' please consult the vignette of this package. The normalization takes about 10
#' hours without parallelization. To save time, this process can be easily
#' accelerated on a computer cluster.
#' @title Normalize CEL Files
#' @param chiptype_list list, storing CEL files in each chiptype
#' @param batchsize number of CEL files to import and to normalize at once
#' @param rerun TRUE or FALSE, whether to run the function
#' @return Files storing normalized expression values in matrix, here one matrix file 
#' per chip type
#' @importFrom affy ReadAffy
#' @importFrom affy mas5
#' @importFrom affy exprs 
#' @examples 
#' # chiptype_list <- split(names(chiptype), as.character(chiptype))
#' normalizeCel(chiptype_list, rerun=FALSE)
#' @export
normalizeCel <- function(chiptype_list, batchsize=300, rerun=TRUE) {
    if(rerun) {
        for(i in names(chiptype_list)) {
            celfiles <- chiptype_list[[i]]
            cel_list <- suppressWarnings(
                split(celfiles, rep(seq_len(ceiling(length(celfiles)/batchsize)), each=batchsize)))
            chipdir <- paste0("./data/", i)
            if(dir.exists(chipdir)) unlink(chipdir, recursive=TRUE)
            dir.create(chipdir)
            mydir <- getwd()
            setwd(chipdir)
            ## Function to run MAS5 in batch
            f <- function(x, cel_list, batchsize) {
                dir.create(paste0("cellbatch_", x))
                mydata <- affy::ReadAffy(filenames=cel_list[[x]], celfile.path="../CEL")
                eset <- affy::mas5(mydata)
                # eset_pma <- affy::mas5calls(mydata) # Generates MAS 5.0 P/M/A calls.
                write.table(affy::exprs(eset), file=paste0("cellbatch_", x, "/mas5exprs.xls"), 
                            quote=FALSE, sep="\t", col.names = NA) 
                }
            lapply(names(cel_list), f, cel_list=cel_list, batchsize=batchsize)
            setwd(mydir)
        }
    } else {
        print("To execute function, set 'rerun=TRUE'")
    }
}

---
title: "_signatureSearchData_: Reference Data for Gene Expression Signature Searching"
author: "Authors: Yuzhu Duan, Dan S. Evans, Richard A. Miller, Nicholas J. Schork, Steven R. Cummings and Thomas Girke"
date: "Last update: `r format(Sys.time(), '%d %B, %Y')`" 
output:
  html_document:
    toc: true
    toc_float:
        collapsed: true
        smooth_scroll: true
    toc_depth: 4
    fig_caption: yes
    code_folding: show
    number_sections: true
  pdf_document:
    toc: true
fontsize: 15pt
bibliography: bibtex.bib
vignette: >
  %\VignetteIndexEntry{signatureSearchData}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

<style>
pre code, pre, code {
  white-space: pre !important;
  overflow-x: auto !important;
  word-break: keep-all !important;
  word-wrap: initial !important;
}
</style>

```{r setup, echo=FALSE, messages=FALSE, warnings=FALSE}
suppressPackageStartupMessages({
    library(signatureSearchData)
})
# knitr::opts_knit$set(root.dir = "~/insync/project/longevityTools_eDRUG/")
```

# Introduction

The `signatureSearchData` package provides access to the reference data used by 
the associated `signatureSearch` software package [@Duan2020-bj]. 
The latter allows to search with a query gene expression signature (GES) a 
database of treatment GESs to identify cellular states sharing similar expression 
responses (connections). This way one can identify drugs or gene knockouts that 
induce expression phenotypes similar to a sample of interest. The resulting 
associations may lead to novel functional insights how perturbagens of interest 
interact with biological systems. 

Currently, `signatureSearchData` includes GES data from the CMap (Connectivity
Map) and LINCS (Library of Network-Based Cellular Signatures) projects that are
largely based on drug and genetic perturbation experiments performed on
variable numbers of human cell lines [@Lamb2006-du; @Subramanian2017-fu]. In
`signatureSearchData` these data sets have been preprocessed to be compatible
with the different gene expression signature search (GESS) algorithms
implemented in `signatureSearch`. The preprocessed data types include but are
not limited to normalized gene expression values (_e.g._ intensity values), log
fold changes (LFC) and Z-scores, p-values or FDRs of differentially expressed
genes (DEGs), rankings based on selected preprocessing routines or sets of top
up/down-regulated DEGs. 

The CMap data were downloaded from the [CMap project
site](https://portals.broadinstitute.org/cmap/) (Version build02). The latter is
a collection of over 7,000 gene expression profiles (signatures) obtained
from perturbation experiments with 1,309 drug-like small molecules on five
human cancer cell lines. The Affymetrix Gene Chip technology was used to
generate the CMAP2 data set. 

In 2017, the LINCS Consortium generated a similar but much larger data set where
the total number of gene expression signatures was scaled up to over one
million. This was achieved by switching to a much more cost effective gene
expression profiling technology called L1000 assay [@Peck2006-rf;
@Edgar2002-di]. The current set of perturbations covered by the LINCS data set
includes 19,811 drug-like small molecules applied at variable concentrations
and treatment times to ~70 human non-cancer (normal) and cancer cell lines.
Additionally, it includes several thousand genetic perturbagens composed of
gene knockdown and over-expression experiments. 

In 2020, the LINCS 2017 database was expanded to a new beta release,
here refered to as LINCS2. It contains >80k perturbations and >200 cell lines and 
over 3M gene expression profiles. This represents roughly a 3-fold expansion of 
the LINCS 2017 database, and several new data sets including CRISPSR knockouts 
of >5k genes and hematopoietic and non-cancer cell models. 
The new LINCS2 datasets can be downloaded from the [clue.io](https://clue.io/releases/data-dashboard) 
site.

The data structures and search algorithms used by `signatureSearch` and
`signatureSearchData` are designed to work with most genome-wide expression
data including hybridization-based methods, such as Affymetrix or L1000, as
well as sequencing-based methods, such as RNA-Seq. Currently,
`signatureSearchData` does not include preconfigured RNA-Seq reference data mainly 
due to the lack of large-scale perturbation studies (_e.g._ drug-based) available in the public
domain that are based on RNA-Seq. This situation may change in the near future
once the technology has become more affordable for this purpose. 


# Install and Load Package

`signatureSearchData` is a R/Bioconductor package and can be installed using 
`BiocManager::install()`.

```{r install, eval=FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("signatureSearchData")
```

After the package is installed, it can be loaded in an R session as follows.
```{r load, eval=FALSE}
library(signatureSearchData)
```

# Explore Data Sets

A summary of the data sets provided by the `signatureSearchData` package can be
obtained with the `query` function of the `ExperimentHub` package. The information
is stored in an object of class `ExperimentHub`, here assigned to `ssd`.

```{r eh_explore_ssd1, eval=TRUE, warning=FALSE, message=FALSE }
library(ExperimentHub)
eh <- ExperimentHub()
ssd <- query(eh, c("signatureSearchData"))
ssd
```

The titles of the data sets can be returned with `ssd$title`.

```{r eh_explore_ssd2, eval=TRUE, warning=FALSE, message=FALSE }
ssd$title
```

More detailed information about each data set can be returned as a `list`, below 
subsetted to 10th entry with `[10]`.

```{r eh_explore_ssd3, eval=TRUE}
as.list(ssd)[10]
```

Details about the usage of `ExperimentHub` can be found in its vignettes [here](http://bioconductor.org/packages/ExperimentHub/).

# LINCS Signature Database

The L1000 assay, used for generating the LINCS data, measures the expression 
of 978 landmark genes and 80 control genes by loading amplified mRNA populations 
onto beads and then detecting their abundance with a fluorescent-based method [@Peck2006-rf]. 
The expression of 11,350 additional genes is imputed from the landmark genes by using 
as training data a large collection of Affymetrix gene chips [@Edgar2002-di].

The LINCS data have been pre-processed by the Broad Institute to 5 different levels
and are available for download from GEO. Level 1 data are the raw mean
fluorescent intensity values that come directly from the Luminex scanner. Level
2 data are the expression intensities of the 978 landmark genes. They have been
normalized and used to impute the expression of the additional 11,350 genes,
forming Level 3 data. A robust z-scoring procedure was used to generate
differential expression values from the normalized profiles (Level 4).
Finally, a moderated z-scoring procedure was applied to the replicated samples
of each experiment (mostly 3 replicates) to compute a weighted average
signature (Level 5). For a more detailed description of the preprocessing
methods used by the LINCS project, readers want to refer to the [LINCS user
guide](https://docs.google.com/document/d/1q2gciWRhVCAAnlvF2iRLuJ7whrGP6QjpsCMq1yWz7dU/edit#).

Disregarding replicates, the LINCS data set contains 473,647 signatures with
unique cell type and treatment combinations. This includes 19,811 drug-like
small molecules tested on different cell lines at multiple concentrations and
treatment times. In addition to compounds, several thousand genetic
perturbations (gene knock-downs and over expressions) have been tested.
Currently, the data described in this vignette are restricted to signatures of
small molecule treatments across different cells lines. However, users have the
option to assemble any custom collection of the LINCS data. For consistency,
only signatures at one specific concentration (10$\mu$M) and one time point
(24h) have been selected for each small molecule in the default collection.
These choices are similar to the conditions used in primary high-throughput
compound screens of cell lines.  Since the selected compound concentrations and
treatment duration have not been tested by LINCS across all cell types yet, a
subset of compounds had to be selected that best met the chosen treatment
requirements. This left us with 8,104 compounds that were uniformly tested at
the chosen concentration and treatment time, but across variable numbers of
cell lines. The total number of expression signatures meeting this requirement
is 45,956, while the total number of cell lines included in this data set is
30. 

## Z-scores from `ExperimentHub`

The LINCS sub-dataset, filtered and assembled according to the above criteria,
can be downloaded from Bioconductor's `ExperimentHub` as HDF5 file. In the
example below, the path to this file is assigned to a character vector called
`lincs_path`. A summary of the content of the HDF5 file can be returned with
the `h5ls` function. Note, due to the large size of the LINCS data set, its download 
takes too much time to evaluate the following code section during the build time of 
this vignette. 

```{r lincs, eval=FALSE}
library(ExperimentHub); library(rhdf5)
eh <- ExperimentHub()
query(eh, c("signatureSearchData", "lincs"))
lincs_path <- eh[['EH3226']]
rhdf5::h5ls(lincs_path) 
```

In this case the loaded data instance includes moderated Z-scores from DE
analyses of 12,328 genes for 8,140 compound treatments across a total of 30
cell lines corresponding to 45,956 expression signatures. This data set can be
used by all set-based and correlation-based GESS methods provided by the 
`signatureSearch` package.

## Z-scores from GEO 

The following explains how to generate the above LINCS data object from
scratch. This also illustrates how to filter the LINCS level 5 data in other
ways.

### Download Level 5 Data 

Download and unzip the following files from GEO entry [GSE92742](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742):

+ <tt>GSE92742_Broad_LINCS_Level5_COMPZ.MODZ_n473647x12328.gctx.gz</tt>
+ <tt>GSE92742_Broad_LINCS_gene_info.txt.gz</tt>
+ <tt>GSE92742_Broad_LINCS_sig_info.txt.gz</tt>

The following code examples assume that the downloaded datasets are stored in a
sub-directory called `data`. All paths in this vignette are given relative to
the present working directory of a user's R session.

### Filter Signatures 

The following selects LINCS Level 5 signatures of compound treatments at a 
concentration of 10$\mu$M and a treatment time of 24 hours. Note, the import command
below may issue a warning message that can be ignored. 

```{r filter_meta42, eval=FALSE}
meta42 <- readr::read_tsv("./data/GSE92742_Broad_LINCS_sig_info.txt") 
dose <- meta42$pert_idose[7]
## filter rows by 'pert_type' as compound, 10uM concentration, and 24h treatment time
meta42_filter <- sig_filter(meta42, pert_type="trt_cp", dose=dose, time="24 h") # 45956 X 14
```

### Z-score Data in HDF5

Next, the large Z-score matrix of expression signatures is imported step-wise
in subsets of manageable sizes and then appended to an HDF5 file (here
`lincs.h5`).  In this vignette, the latter is referred to as the LINCS
Z-score database. Since the size of the full matrix is several GBs in size, it would
consume too much memory to be read into R at once. Reading the matrix in
smaller batches and appending them to an HDF5 file is much more memory
efficient. Subsequently, the `HDF5Array` function from the `HDF5Array` package
combined with the `SummarizedExperiment` function
could import the data from the HDF5 file into a `SummarizedExperiment` object, 
here assigned to `se`.

```{r extract_modz, eval=FALSE}
library(signatureSearch)
gctx <- "./data/GSE92742_Broad_LINCS_Level5_COMPZ.MODZ_n473647x12328.gctx"
gctx2h5(gctx, cid=meta42_filter$sig_id, new_cid=meta42_filter$pert_cell_factor,
        h5file="./data/lincs.h5", by_ncol=5000, overwrite=TRUE)
library(HDF5Array)
se <- SummarizedExperiment(HDF5Array("./data/lincs.h5", name="assay"))
rownames(se) <- HDF5Array("./data/lincs.h5", name="rownames")
colnames(se) <- HDF5Array("./data/lincs.h5", name="colnames")
```

### DEG and Cutoff Definitions
The DEGs for the LINCS level 5 Z-score database can be defined by users by setting
the cutoffs of Z-scores (*e.g.* +2 and -2) to define up/down regulated DEGs. 
The cutoff parameters of defining DEGs are available as the argument of the 
GESS methods when the reference database needs to be DEG sets and the `lincs`
Z-score data are provided (only for _gCMAP_ and _Fisher_ GESS methods). 
The query gene sets could also be defined by users by either selecting 150 up and down genes
or defining cutoffs of Z-scores. The query gene sets can be used for _CMAP_, 
_LINCS_ GESS methods. The following codes show examples of defining DEGs used as 
query and defining DEG sets used as reference database.

Defining query gene sets
```{r lincs_degs, eval=FALSE}
library(signatureSearch)
# Get up and down 150 DEGs
degs <- getDEGSig(cmp="vorinostat", cell="SKB", refdb="lincs", Nup=150, Ndown=150)

# Get DEGs by setting cutoffs
degs2 <- getDEGSig(cmp="vorinostat", cell="SKB", refdb="lincs", higher=2, lower=-2)
```

Defining gene sets reference database. The LINCS Z-score reference database will 
be internally converted to the gene sets database in forms of the 0, 1, -1 matrix when 
user defining the `higher` and `lower` cutoffs in the `gess_gcmap` and `gess_fisher`
functions.
```{r lincs_degs_db, eval=FALSE}
# gCMAP method
gep <- getSig("vorinostat", "SKB", refdb="lincs")
qsig_gcmap <- qSig(query = gep, gess_method = "gCMAP", refdb = "lincs")
gcmap_res <- gess_gcmap(qsig_gcmap, higher=2, lower=-2)
# Fisher method
qsig_fisher <- qSig(query = degs, gess_method = "Fisher", refdb = "lincs")
fisher_res <- gess_fisher(qSig=qsig_fisher, higher=2, lower=-2)
```

## Intensities from ExperimentHub
The LINCS Level 3 data can be downloaded from
[GEO](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742) the same way
as described above for the Level 5 data. The Level 3 data contain normalized
gene expression values across all treatments and cell lines used by LINCS.  The
Level 3 signatures were filtered using the same dosage and duration criteria as
the Level 5 data. The biological replicate information included in the Level 3
data were collapsed to mean values.  Subsequently, the resulting matrix of mean
expression values was written to an HDF5 file. The latter is referred to as
`lincs_expr` database containing 38,824 signatures for a total of 5,925 small
molecule treatments and 30 cell lines. Although the LINCS Level 3 and 5 data
are filtered here the same way, the number of small molecules represented in
the Level 3 data (5,925) is smaller than in the Level 5 data (8,140). The reason for
this inconsistency is most likely that the Level 3 dataset, downloadable from GEO,
is incomplete.

The filtered and processed LINCS Level3 data (`lincs_expr`) can be loaded from 
Bioconductor's `ExperimentHub` interface as follows.

```{r lincs_expr, eval=FALSE}
library(ExperimentHub)
eh <- ExperimentHub()
query(eh, c("signatureSearchData", "lincs_expr"))
lincs_expr_path <- eh[['EH3227']]
```

In this case the loaded `lincs_expr` instance includes mean expression values of 
12,328 genes for 5,925 compound treatments across a total of 30 cell lines. 
This data set can be used by all correlation-based GESS methods provided by the 
`signatureSearch` package.

## Intensities from GEO

The following steps explain how to generate the above data set from scratch. 
This also illustrates how to filter the LINCS Level 3 data in other ways.

### Download Level 3 Data 

Download and unzip the following files from GEO entry [GSE92742](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742):

+ <tt>GSE92742_Broad_LINCS_Level3_INF_mlr12k_n1319138x12328.gctx.gz</tt>
+ <tt>GSE92742_Broad_LINCS_gene_info.txt.gz</tt>
+ <tt>GSE92742_Broad_LINCS_inst_info.txt.gz</tt>

As above, the following code examples assume that the downloaded datasets are
stored in a sub-directory called `data`. All paths in this vignette are given
relative to the present working directory of a user's R session.

### Filter Signatures 

The following selects LINCS Level 3 signatures of compound treatments at a 
concentration of 10$\mu$M and a treatment time of 24 hours. 

```{r filter_expr, eval=FALSE}
inst42 <- readr::read_tsv("./data/GSE92742_Broad_LINCS_inst_info.txt") 
inst42_filter <- inst_filter(inst42, pert_type="trt_cp", dose=10, dose_unit="um", 
                             time=24, time_unit="h") # 169,795 X 13
```

### Mean Intensities in HDF5

Next, mean expression values are calculated among biological replicates and then appended
in batches to the corresponding HDF5 file.

```{r extract_expr, eval=FALSE}
# It takes some time
library(signatureSearch)
meanExpr2h5(gctx="./data/GSE92742_Broad_LINCS_Level3_INF_mlr12k_n1319138x12328.gctx",
            inst=inst42_filter, h5file="./data/lincs_expr.h5") # 12328 X 38824
```

# LINCS2 Signature Database
The LINCS 2020 beta release data set contains 1.2 million signatures with 720,216
compound treatment GESs of 34,418 compounds, 34,171 gene over-expression on 4,040 genes, 
318,208 gene knockdowns on 7,976 genes using shRNAs on 4,917 genes (177263) and 
CRISPR on 5,158 genes (140945). Out of 720,216 compound treatments, it 
includes 34,418 drug-like small molecules tested on 230 different cell lines at 
multiple concentrations and treatment times. To minimize redundancy of perturbagens
having many signatures in different cell lines, dosage and treatment times, 
the 'exemplar' signature for each perturbagen in selected cell lines was
assembled. These signatures are annotated from CLUE group and are generally 
picked based on TAS (Transcriptional Activity Score), such that the signature 
with the highest TAS is chosen as exemplar. The generated LINCS2 dataset contains
moderated z-scores from DE analysis of 12,328 genes from 30,456 compound treatments 
of 58 cell lines corresponding to a total of 136,460 signatures. It is exactly 
the same as the reference database used for the Query Tool in CLUE website.
Like the LINCS database, users have the option to assemble any custom collection 
from the original LINCS 2020 beta release dataset.

## Z-scores from `ExperimentHub`

The LINCS2 database can be downloaded from Bioconductor's `ExperimentHub` as 
HDF5 file. In the example below, the path to this file is assigned to a character 
vector called `lincs2_path`. A summary of the content of the HDF5 file can be 
returned with the `h5ls` function. Note, due to the large size of the LINCS 
data set, its download takes too much time to evaluate the following code 
section during the build time of this vignette. 

```{r lincs2, eval=FALSE}
library(ExperimentHub); library(rhdf5)
eh <- ExperimentHub()
query(eh, c("signatureSearchData", "lincs2"))
lincs2_path <- eh[['EH7297']]
rhdf5::h5ls(lincs2_path) 
```

In this case the loaded data instance includes moderated Z-scores from DE
analyses of 12,328 genes for 30,456 compound treatments across a total of 58
cell lines corresponding to 136,460 expression signatures. This data set can be
used by all set-based and correlation-based GESS methods provided by the 
`signatureSearch` package.

## Z-scores from CLUE 

The following explains how to generate the above LINCS2 data object from
scratch.

### Download Level 5 Data 

Download level 5 data for compounds from [CLUE](https://clue.io/releases/data-dashboard) as a gctx file:

+ <tt>level5_beta_trt_cp_n720216x12328.gctx</tt>

In the example below, examplar signatures are identified by downloading the meta data and selecting 
records with `is_exemplar_sig` equal to one. These records are saved to an object called `exemplar` which is used to generate compound IDs and specify which records in the the level 5 data are imported stepwise and appended to an HDF5 file (here `lincs2.h5` referred to as LINCS2 in this vignette). Nesting the `SummarizedExperiment` and `HDF5Array` functions can load LINCS2 into a summarized experiment object called `sedb` that can be used with the `signatureSearch` package.

```{r lincs2_hdf5, eval=FALSE}
siginfo_beta <- fread("https://s3.amazonaws.com/macchiato.clue.io/builds/LINCS2020/siginfo_beta.txt")
exemplar <- siginfo_beta %>% filter(pert_type=="trt_cp" & is_exemplar_sig == 1)
new_cid <- paste(exemplar$pert_id, exemplar$cell_iname, rep("trt_cp", length(exemplar$cmap_name)), sep="__")
gctx2h5("level5_beta_trt_cp_n720216x12328.gctx", cid=exemplar$sig_id, new_cid=new_cid,
        h5file="lincs2.h5", by_ncol=5000, overwrite=TRUE)
DBpath <- "lincs2.h5"
sedb <- SummarizedExperiment(HDF5Array(DBpath, name="assay"))
rownames(sedb) <- HDF5Array(DBpath, name="rownames")
colnames(sedb) <- HDF5Array(DBpath, name="colnames")
```

# CMap2 Signature Database

CMap2 (Version build02) contains GESs for 1,309 drugs and eight cell lines that
were generated with Affymetrix Gene Chips as expression platform. In some cases
this includes drug treatments at different concentrations and time points.  For
consistency, the CMap2 data was reduced to drug treatments with concentrations
and time points that are comparable to those used for the above LINCS data.
CMap2 data can be downloaded from GEO or its project site either in raw format or
as rank transformed matrix.  The ranks are based on DEG analyses of drug
treatments (drug vs. no-drug) where the resulting Z-scores were used to
generate the rank matrix. The latter was used here and is referred to as
`rankMatrix`. The Affymetrix probe set identifiers stored in the row name slot 
of this matrix were translated into gene identifies.  To obtain a matrix with unique gene
identifiers, the ranks for genes represented by more than one probe set were
averaged and then re-ranked accordingly. This final gene level rank matrix, referred to as `cmap_rank`,
contains rank profiles for 12,403 genes from 1,309 compound treatments in up to 5
cells corresponding to a total of 3,587 treatment signatures. This matrix can
be used for all GESS methods in the `signatureSearch` package that are
compatible with rank data, such as the `gess_cmap` method.

## Rank Matrix from `ExperimentHub`

The `cmap_rank` data can be downloaded from Bioconductor's `ExperimentHub` as HDF5
file. Since CMap2 is much smaller than LINCS, it can be imported in its
entirety into a `SummarizedExperiment` object (here assigned to `se`) without
excessive memory requirements. 

```{r cmap_rank, eval=FALSE}
library(ExperimentHub)
eh <- ExperimentHub()
query(eh, c("signatureSearchData", "cmap_rank"))
cmap_rank_path <- eh[["EH3225"]]
se <- SummarizedExperiment(HDF5Array(cmap_rank_path, name="assay"))
rownames(se) <- HDF5Array(cmap_rank_path, name="rownames")
colnames(se) <- HDF5Array(cmap_rank_path, name="colnames")
```

## Rank Matrix from Sources

The following steps explain how to generate the above CMap2 rank data set from scratch. 

### Download Rank Data 

The `rankMatrix` can be downloaded from the CMap project site [here](https://portals.broadinstitute.org/cmap).
The specific file to download from this site is [rankMatrix.txt.zip](ftp://ftp.broad.mit.edu/pub/cmap/rankMatrix.txt.zip).
As before, it should be saved and unzipped in the `data` directory of a user's R session.

### Filter instances

The following selects from `rankMatrix` for each compound the chosen treatment
concentration and time point. This is achieved with help of the experiment
annotation file `cmap_instances_02.txt`, also available from the CMap project
site. Since this file is relatively small it has been included in the
`signatureSearchData` package from where it can be loaded into R as shown
below.

```{r filter_rankm, eval=FALSE}
path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
cmap_inst <- read.delim(path, check.names=FALSE) 
inst_id <- cmap_inst$instance_id[!duplicated(paste(cmap_inst$cmap_name, cmap_inst$cell2, sep="_"))]
rankM <- read.delim("./data/rankMatrix.txt", check.names=FALSE, row.names=1) # 22283 X 6100
rankM_sub <- rankM[, as.character(inst_id)]
colnames(rankM_sub) <- unique(paste(cmap_inst$cmap_name, cmap_inst$cell2, "trt_cp", sep="__"))
```

### Annotated Gene Level Data

#### Annotation Information

The following generates annotation information for Affymetirx probe set
identifiers. Note, the three different Affymetrix chip types (HG-U133A,
HT_HG-U133A, U133AAofAv2) used by CMap2 share nearly all probe set identifiers,
meaning it is possible to use the same annotation package (here `hgu133a.db`)
for all three.

```{r affyid_annot, eval=FALSE, message=FALSE}
library(hgu133a.db)
myAnnot <- data.frame(ACCNUM=sapply(contents(hgu133aACCNUM), paste, collapse=", "), 
                      SYMBOL=sapply(contents(hgu133aSYMBOL), paste, collapse=", "), 
                      UNIGENE=sapply(contents(hgu133aUNIGENE), paste, collapse=", "), 
                      ENTREZID=sapply(contents(hgu133aENTREZID), paste, collapse=", "), 
                      ENSEMBL=sapply(contents(hgu133aENSEMBL), paste, collapse=", "), 
                      DESC=sapply(contents(hgu133aGENENAME), paste, collapse=", "))
saveRDS(myAnnot, "./data/myAnnot.rds")
```

#### Gene Level Data

The `probe2gene` function transforms probe set to gene level data. If genes are
represented by several probe sets then their mean intensities are used.

```{r mr_prob, eval=FALSE}
rankM_sub_gene <- probe2gene(rankM_sub, myAnnot) 
```

### Store Rank Matrix in HDF5 file

The sub-setted `rankMatrix` is written to an HDF5 file, referred to as
`cmap_rank` database.

```{r cmap_rank2h5, eval=FALSE}
matrix2h5(rankM_sub_gene, "./data/cmap_rank.h5", overwrite=TRUE) # 12403 X 3587
rhdf5::h5ls("./data/cmap_rank.h5")
## Read in cmap_rank.h5 as SummarizedExperiment object
se <- SummarizedExperiment(HDF5Array("./data/cmap_rank.h5", name="assay"))
rownames(se) <- HDF5Array("./data/cmap_rank.h5", name="rownames")
colnames(se) <- HDF5Array("./data/cmap_rank.h5", name="colnames")
```

## Intensities from `ExperimentHub`

To search CMap2 with `signatureSearch's` correlation based GESS methods
(`gess_cor`), normalized gene expression values (here intensities) are required
where the biological replicate information has been collapsed to mean values.
For this, the `cmap_expr` database has been created from CEL files, which are
the raw data of the Affymetrix technology. To obtain normalized expression
data, the CEL files were downloaded from the [CMap project site](https://portals.broadinstitute.org/cmap/#), and then
processed with the MAS5 algorithm. Gene level expression data was generated the
same way as described above. Next, the gene expression values for different
concentrations and treatment times of each compound and cell were averaged.
Subsequently, the expression matrix was saved to an HDF5 file, 
referred to as the `cmap_expr` database. It represents mean expression values
of 12,403 genes for 1,309 compound treatments in up to 5 cells (3,587 signatures in total).

The `cmap_expr` database can be downloaded as HDF5 file from Bioconductor's 
`ExperimentHub` as follows.

```{r cmap_expr, eval=FALSE}
library(ExperimentHub)
eh <- ExperimentHub()
query(eh, c("signatureSearchData", "cmap_expr"))
cmap_expr_path <- eh[["EH3224"]]
```

This data set can be used by all correlation-based GESS methods provided by the
`signatureSearch` package.

## Intensities from Sources
How to generate the above `cmap_expr` database from scratch is explained in the Supplementary Material 
section of this vignette (see Section 8). 


# Custom Signature Databases

Custom databases of GESs can be built with the `build_custom_db` function
provided by the `signatureSearch` package. For this the user provides custom
genome-wide gene expression data (e.g. for drug, disease or genetic
perturbations) in a `data.frame` or `matrix`. The gene expression data can be
most types of the pre-processed gene expression values described under section
1.4 of the `signatureSearch` vignette.


# Additional Datasets

The `signatureSearchData` package also contains several annotation datasets, such
as drug-target information of small molecules. They are 
required for `signatureSearch's` functional enrichment analysis (FEA) routines. 
Currently, most of these annotation data were downloaded from the following databases:

+ [DrugBank](https://www.drugbank.ca/) 
+ [CLUE](https://clue.io/) 
+ [STITCH](http://stitch.embl.de/)

# Supplemental Material

## CMap2 Intensities from Sources
The following steps explain how to generate the `cmap_expr` database in subsection 5.3 from scratch.
They are intended for expert users and have been included here for reproduciblity reasons.

### Directory Structure

The large number of files processed in the next steps are organized in two
sub-directories of a user's R session. Input files will be stored in a `data`
directory, while all output files will be written to a `results` directory. 

```{r work_envir, eval=FALSE}
dir.create("data"); dir.create("data/CEL"); dir.create("results") 
```

### Download of CEL Files

The `getCmapCEL` function will download the 7,056 CEL files from the [CMap
project site](http://www.broadinstitute.org/cmap), and save each of them to a
subdirectory named `CEL` under `data`. Since this download step will take some time,
the argument `rerun` has been assigned `FALSE` in the below function call to
avoid running it accidentally. To execute the download, the argument `rerun`
needs to be assigned `TRUE`. If the raw data are not needed, users can skip
this time consuming step and work with the preprocessed `cmap_expr` database
downloaded from the `ExperimentHub` instead.

```{r download_cmap, eval=FALSE}
getCmapCEL(rerun=FALSE) 
```

### Determine Chip Type 
The CMAP data set is based on three different Affymetrix chip types (HG-U133A,
HT_HG-U133A and U133AAofAv2). The following extracts the chip type information
from the downloaded CEL files and stores the information in an `rds` file with the path 
`./data/chiptype.rds`.

```{r get_cel_type, eval=FALSE}
library(affxparser)
celfiles <- list.files("./data/CEL", pattern=".CEL$")
chiptype <- sapply(celfiles, function(x) affxparser::readCelHeader(paste0("data/CEL/", x))$chiptype)
saveRDS(chiptype, "./data/chiptype.rds")
```

### Normalization of CEL Files

The following processes the CEL files from each chip type separately using the
MAS5 normalization algorithm. The results will be written to 3 subdirectores
under `data` that are named after the chip type names. To reduce the memory
consumption of this step, the CEL files are normalized in batches of 200. The
normalization takes about 10 hours without parallelization.  To save time, this
process can be easily accelerated on a computer cluster.

```{r normalize_chips, eval=FALSE}
chiptype <- readRDS("./data/chiptype.rds")
chiptype_list <- split(names(chiptype), as.character(chiptype))
normalizeCel(chiptype_list, batchsize=200, rerun=FALSE) 
```

Next the results from each chip type are assembled in a data frame. After this
all three of these data frames are combined to a single one, here named `mas5df`.
```{r comb_chip_type_data, eval=FALSE}
chiptype_dir <- unique(chiptype)
combineResults(chiptype_dir, rerun=FALSE)
mas5df <- combineNormRes(chiptype_dir, norm_method="MAS5")
```

### Gene Level Data
After moving the `myAnnot.rds` file from above into the `data` directory, the `probe2gene`
function is used to transforms probe set to gene level data. If genes are represented by
several probe sets then their mean intensities are used.

```{r prof2gene, eval=FALSE}
myAnnot <- readRDS("./data/myAnnot.rds") 
mas5df <- probe2gene(mas5df, myAnnot) 
saveRDS(mas5df,"./data/mas5df.rds")
```

### Average Intensities Across Replicates

The following averages the normalized gene expression values for different
concentrations, treatment times and replicates of compounds and cell types.
```{r rma2cmap_expr, eval=FALSE}
mas5df <- readRDS("./data/mas5df.rds") # dim: 12403 x 7056
path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
cmap_inst <- read.delim(path, check.names=FALSE) 
cmap_drug_cell_expr <- meanExpr(mas5df, cmap_inst) # dim: 12403 X 3587
saveRDS(cmap_drug_cell_expr, "./data/cmap_drug_cell_expr.rds")
```

### Mean Intensities in HDF5

The normalized and averaged expression values are saved to an HDF5 file,
referred to as `cmap_expr` database.

```{r gen_cmap_expr, eval=FALSE}
cmap_drug_cell_expr <- readRDS("./data/cmap_drug_cell_expr.rds")
## match colnames to '(drug)__(cell)__(factor)' format
colnames(cmap_drug_cell_expr) <- gsub("(^.*)_(.*$)", "\\1__\\2__trt_cp", 
                                      colnames(cmap_drug_cell_expr)) 
matrix2h5(cmap_drug_cell_expr, "./data/cmap_expr.h5", overwrite=TRUE)
h5ls("./data/cmap_expr.h5")
```

## CMap2 LFC database

The MAS5 normalized CEL files from the `CMap2 Intensities from Sources` section
can be used for DE analysis with `limma` package to get the `logMA` matrix 
containing the LFC scores. The treatment v.s. control instances 
were defined in the `cmap_instances_02.txt`. The same as the `cmap_expr` database,
only one treatment condition is selected for a compound in a cell. 
So, the resulting `logMA` matrix has LFC scores of 1,281 compound treatments in 5 cells
(3,478 signatures in total). The latter was stored in an HDF5 file, which is 
referred to as the `cmap` database. Note, The number of compound treatments in `cmap`
database is slightly different from that of the `cmap_expr` database. The reason
is that some of the compound treatment is discarded if the number of control and treatment
samples are less than 3 during the DE analysis.

### Load from `ExperimentHub`
The preprocessed `cmap` database can be loaded through the `ExperimentHub` interface as follows.
```{r cmap, eval=FALSE}
library(ExperimentHub)
eh <- ExperimentHub()
query(eh, c("signatureSearchData", "cmap"))
cmap_path <- eh[["EH3223"]]
```

In summary, the loaded data instance includes LFC scores of 12,403
genes of 1,281 compound treatments across a total of 5 cell lines. This data set 
can be used by all GESS methods provided by the `signatureSearch` package.

### Generate from Sources
The following steps explain how to generate the above data set from the MAS5 
normalized expression matrix of CEL files (`mas5df`) generated at the
`CMap2 Intensities from Sources` section. Use the same working directory as 
`cmap_expr` signature database.

#### Generate list of CEL files defining treatment vs. control comparisons

The `sampleList` function extracts the sample comparisons (contrasts) from the
CMAP annotation table and stores them as a list.
```{r cel_file_list, eval=FALSE}
path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
cmap_inst <- read.delim(path, check.names=FALSE) 
comp_list <- sampleList(cmap_inst, myby="CMP_CELL")
```

#### DEG analysis with `limma`
The analysis of differentially expressed genes (DEGs) is performed with the `limma` package.
```{r deg_limma, eval=FALSE}
mas5df <- readRDS("./data/mas5df.rds")
degList <- runLimma(df=log2(mas5df), comp_list, fdr=0.10, foldchange=1, verbose=TRUE)
saveRDS(degList, "./results/degList.rds") # saves entire degList
```

#### Save the LFC and FDR matrix to an HDF5 file
The `logMA` contains the LFC scores of compound treatments in cells. The LFC
as well as the FDR matrix are saved to an HDF5 file, which is the `cmap` database.
```{r se, eval=FALSE}
degList <- readRDS("./results/degList.rds")
logMA <- degList$logFC
## match colnames of logMA to '(drug)__(cell)__(factor)' format
colnames(logMA) <- gsub("(^.*)_(.*$)", "\\1__\\2__trt_cp", colnames(logMA))
fdr <- degList$FDR
colnames(fdr) <- gsub("(^.*)_(.*$)", "\\1__\\2__trt_cp", colnames(fdr))
matrix2h5(logMA, "./data/cmap.h5", name="assay", overwrite=TRUE) # 12403 X 3478
matrix2h5(fdr, "./data/cmap.h5", name="padj", overwrite=FALSE)
rhdf5::h5ls("./data/cmap.h5")
```

#### DEG and Cutoff Definitions
The DEGs for the CMAP2 database can be defined by users by setting
the cutoffs of LFC as well as the adjusted p-value or FDR to define up/down 
regulated DEGs if the p-value matrix is available in the CMAP HDF5 file. 
The cutoff parameters of defining DEGs are available as the argument of the 
GESS methods when the reference database needs to be DEG sets (only for _gCMAP_ 
and _Fisher_ GESS methods). 

The query gene sets could also be defined by users by either selecting 150 up and down genes
or defining cutoffs of LFC and FDRs. The query gene sets can be used for _CMAP_, _LINCS_ GESS
methods. The following codes show examples of defining DEGs used as query and
defining DEG sets used as reference database.

Defining query gene sets
```{r cmap_degs, eval=FALSE}
library(signatureSearch)
# Get up and down 150 DEGs
degs <- getDEGSig(cmp="vorinostat", cell="PC3", refdb="cmap", Nup=150, Ndown=150)

# Get DEGs by setting cutoffs
degs2 <- getDEGSig(cmp="vorinostat", cell="PC3", refdb="cmap", 
                 higher=1, lower=-1, padj=0.05)
```

Defining gene sets reference database. The CMAP2 reference database will be internally
converted to the gene sets database in forms of the 0, 1, -1 matrix when 
user defining the `higher`, `lower` and `padj` cutoffs in the `gess_gcmap` and 
`gess_fisher` functions. The `padj` argument is supported when the reference 
database contains both the LFC score and p-value matrix, so it is possible to 
define DEGs combined from the LFC and p-value (could be either p-value,
adjusted p-value or FDR depending on the type of p-value stored in dataset
named as `padj`) cutoffs.
```{r cmap_degs_db, eval=FALSE}
# gCMAP method
gep <- getSig("vorinostat", "PC3", refdb="cmap")
qsig_gcmap <- qSig(query = gep, gess_method = "gCMAP", refdb = "cmap")
gcmap_res <- gess_gcmap(qsig_gcmap, higher=1, lower=-1, padj=0.05)
# Fisher method
qsig_fisher <- qSig(query = degs, gess_method = "Fisher", refdb = "cmap")
fisher_res <- gess_fisher(qSig=qsig_fisher, higher=1, lower=-1, padj=0.05)
```

# Session Info
```{r sessionInfo}
sessionInfo()
```

# Funding

This project is funded by NIH grants
[U19AG02312](https://www.longevityconsortium.org/) and
[U24AG051129](https://www.longevitygenomics.org/) awarded by the National
Institute on Aging (NIA). Subcomponents of the environment are based on methods
developed by projects funded by NSF awards ABI-1661152 and PGRP-1810468. The
High-Performance Computing (HPC) resources used for optimizing and applying the code of
this project were funded by NIH and NSF grants 1S10OD016290-01A1 and MRI-1429826,
respectively. 

# References

\name{GO_DATA_drug}
\alias{GO_DATA_drug}
\title{GO to Drug Annotation Environment} 
\description{
It is an RDS file storing the GO annotation environment where the gene SYMBOLs 
are mapped to drug names via drug-target interactions. The GO to drug annotation
environment contains GO term ID to drug names, drug name to annotated 
GO terms, GO term ID to its description and GO term ID to its ontology mappings.
This environment is used for \code{dsea_*} methods in the \code{signatureSearch} 
package to do GO enrichment analysis directly with test drug set.
This environment is stored in an RDS file to accelerate the speed by avoiding 
building this environment from scratch every time running the \code{dsea} functions. 
}

\seealso{
    \code{dsea}
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
qr <- query(eh, c("signatureSearchData", "GO_DATA"))
go_data_drug <- eh["EH3232"]
# go_data_drug <- eh[["EH3232"]]
}

\keyword{datasets}

\name{cmap_expr}
\alias{cmap_expr}
\title{CMap Intensity Signature Database}
\description{
To search CMap2 with signatureSearch's correlation based GESS methods (gess_cor),
normalized gene expression values (here intensities) are required where the 
biological replicate information has been collapsed to mean values. For this, 
the \code{cmap_expr} database has been created from CEL files, which are the 
raw data of the Affymetrix technology. To obtain normalized expression data,
the CEL files were downloaded from the CMap project site, and then processed 
with the MAS5 algorithm. Gene level expression data was generated the same way 
as described in help file of \code{cmap_rank}. Next, the gene expression values 
for different concentrations and treatment times of each compound and cell 
were averaged. Subsequently, the expression matrix was saved to an HDF5 file, 
referred to as the cmap_expr database. It represents mean expression values of 
12,403 genes for 1,309 compound treatments in up to 5 cells (3,587 signatures in total).
}
\details{
The cmap_expr database can be downloaded as HDF5 file from Bioconductor’s 
ExperimentHub as shown in the Example section. 

The loaded \code{cmap_expr} data object is generated from the raw CEL files
downloaded from the CMap project site.
For documentation and code of generating the cmap_expr databases from sources, 
please refer to the vignette of this package by running 
\code{browseVignettes("signatureSearchData")} in R.
}

\references{
CMap project site: https://portals.broadinstitute.org/cmap
}

\examples{
library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "cmap_expr"))
# cmap_expr_path <- eh[["EH3224"]]
# rhdf5::h5ls(cmap_expr_path)
}

\keyword{datasets}

\name{lincs}
\alias{lincs}
\title{LINCS Z-score Signature Database} 
\description{
The L1000 assay, used for generating the LINCS data, measures the expression of 
978 landmark genes and 80 control genes by loading amplified mRNA populations 
onto beads and then detecting their abundance with a fluorescent-based method. 
The expression of 11,350 additional genes is imputed from the landmark genes 
by using as training data a large collection of Affymetrix gene chips.

The LINCS data have been pre-processed by the Broad Institute to 5 different 
levels and are available for download from GEO. Level 1 data are the raw mean 
fluorescent intensity values that come directly from the Luminex scanner. 
Level 2 data are the expression intensities of the 978 landmark genes. 
They have been normalized and used to impute the expression of the additional 
11,350 genes, forming Level 3 data. A robust z-scoring procedure was used to 
generate differential expression values from the normalized profiles (Level 4). 
Finally, a moderated z-scoring procedure was applied to the replicated samples 
of each experiment (mostly 3 replicates) to compute a weighted average signature 
(Level 5). For a more detailed description of the preprocessing methods used by 
the LINCS project, readers want to refer to the LINCS user guide.

Disregarding replicates, the LINCS data set contains 473,647 signatures with 
unique cell type and treatment combinations. This includes 19,811 drug-like 
small molecules tested on different cell lines at multiple concentrations and 
treatment times. In addition to compounds, several thousand genetic perturbations 
(gene knock-downs and over expressions) have been tested. Currently, the data 
stored in this package is restricted to signatures of small molecule treatments 
across different cells lines. However, users have the option to assemble any 
custom collection of the LINCS data. For consistency, only signatures at one 
specific concentration (10uM) and one time point (24h) have been selected for 
each small molecule in the default collection. These choices are similar to the 
conditions used in primary high-throughput compound screens of cell lines. 
Since the selected compound concentrations and treatment duration have not been 
tested by LINCS across all cell types yet, a subset of compounds had to be 
selected that best met the chosen treatment requirements. This left us with 
8,104 compounds that were uniformly tested at the chosen concentration and 
treatment time, but across variable numbers of cell lines. The total number of
expression signatures meeting this requirement is 45,956, while the total number 
of cell lines included in this data set is 30.
}
\details{
The LINCS sub-dataset, filtered and assembled according to the above criteria, 
can be downloaded from Bioconductor’s \code{ExperimentHub} as HDF5 file. 
In this case the loaded data instance includes moderated Z-scores from 
DE analyses of 12,328 genes for 8,140 compound treatments across a total of 
30 cell lines corresponding to 45,956 expression signatures. This data set can 
be used by all set-based and correlation-based GESS methods provided by the 
signatureSearch package.

The loaded \code{lincs} data object is filtered and generated from the original
LINCS level 5 data stored at GEO:
https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742. 
For documentation and code of generating the lincs databases from sources, 
please refer to the vignette of this package by running 
\code{browseVignettes("signatureSearchData")} in R.
}

\references{
  LINCS source data at 5 levels from GEO: 
  https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742
  
  LINCS User Guide v2.1: 
  https://docs.google.com/document/d/1q2gciWRhVCAAnlvF2iRLuJ7whrGP6QjpsCMq1yWz7dU/edit#
}

\examples{
library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "lincs"))
# lincs_path <- eh[['EH3226']]
# rhdf5::h5ls(lincs_path) 
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineNormRes.R
\name{combineNormRes}
\alias{combineNormRes}
\title{Combine Normalized Data from Different Chip Types}
\usage{
combineNormRes(chiptype_dir, norm_method)
}
\arguments{
\item{chiptype_dir}{character vector containing paths to chiptype directories}

\item{norm_method}{normalization method, one of "MAS5" or "rma".}
}
\value{
Object of class \code{data.frame} containing normalization values
}
\description{
Function assembles normalization results from different chip types
into a single data frame. Note, this is possible since the three chip types
used by the CMap2 project contain nearly identical probe set ids. Typically,
the input for this step is generated by the upstream \code{normalizeCel} and
\code{combineResults} functions. For more context, please consult the vignette
of this package.
}
\examples{
# chiptype_dir <- unique(readRDS("./data/chiptype.rds"))
combineNormRes(chiptype_dir, norm_method="not run")
}

\name{goAnno_drug}
\alias{goAnno_drug}
\title{GO Term ID to Drug Name Mapping Annotation} 
\description{
It is a \code{data.frame} containing the GO term ID to drug name mapping 
information used for \code{dsea_*} functions in \code{signatureSearch} package.
}

\details{
The original GO annotation system contains GO term ID to gene SYMBOL id mappings,
the latter are converted to drug names mappings via
drug to target protein-encoding genes links in DrugBank, CLUE and STITCH databases.
}
\seealso{
  \itemize{
    \item \code{dsea_hyperG}
    \item \code{dsea_GSEA}
  }
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
#goAnno_drug <- eh[["EH3230"]]
}

\keyword{datasets}

\name{cmap}
\alias{cmap}
\title{CMap2 LFC Signature Database}
\description{
The MAS5 normalized CEL files from the help file of \code{\link{cmap_expr}} 
can be used for differential expression (DE) analysis with `limma` package to 
get a matrix containing the LFC (log2 fold change) scores. The columns of the 
matrix are the treatment v.s. control instances, which is defined in the 
'cmap_instances_02.txt' file downloaded from CMap project website. 
The same as the `cmap_rank` database, only one treatment condition is selected 
for a compound in a cell. 
The resulting matrix represents LFC of 12,403 genes for 1,281 compound treatments 
in up to 5 cells (3,478 signatures in total). The latter was stored in an HDF5 
file, which is referred to as the \code{cmap} database. Note, The number of 
compound treatments in \code{cmap} database is slightly different from that of 
the \code{cmap_expr} database. The reason is that some of the compound treatment 
is discarded if the number of control and treatment samples are less than 3 
during the DE analysis.
}
\details{
The cmap database can be downloaded as HDF5 file from Bioconductor’s 
ExperimentHub as shown in the Example section. 

The same as the \code{cmap_expr} dataset, the loaded \code{cmap} data object is 
also generated from the raw CEL files downloaded from the CMap project site and
processed as describe above.
}

\examples{
library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "cmap"))
# cmap_path <- eh[["EH3223"]]
# rhdf5::h5ls(cmap_path)
}

\keyword{datasets}

\name{lincs_expr}
\alias{lincs_expr}
\title{LINCS Intensity Signature Database} 
\description{
The LINCS Level 3 data can be downloaded from GEO the same way as described
for the Level 5 data available at help file of \code{\link{lincs}}. 
The Level 3 data contain normalized gene expression values across all treatments 
and cell lines used by LINCS. The Level 3 signatures were filtered using the 
same dosage and duration criteria as the Level 5 data. The biological replicate 
information included in the Level 3 data were collapsed to mean values. 
Subsequently, the resulting matrix of mean expression values was written to an 
HDF5 file. The latter is referred to as \code{lincs_expr} database containing 
38,824 signatures for a total of 5,925 small molecule treatments and 30 cell lines. 
Although the LINCS Level 3 and 5 data are filtered here the same way, the 
number of small molecules represented in the Level 3 data (5,925) is smaller 
than in the Level 5 data (8,140). The reason for this inconsistency is most 
likely that the Level 3 dataset, downloadable from GEO, is incomplete.
}
\details{
The filtered and processed LINCS Level3 data (lincs_expr) can be loaded from 
Bioconductor’s \code{ExperimentHub} interface as shown in Examples section. 
In this case the loaded lincs_expr instance includes mean expression values of
12,328 genes for 5,925 compound treatments across a total of 30 cell lines
(38,824 signatures in total). 
This data set can be used by all correlation-based GESS methods provided by the
signatureSearch package.

The loaded \code{lincs_expr} data object is filtered and generated from the original
LINCS level 3 data stored at GEO:
https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742. 
For documentation and code of generating the lincs_expr databases from sources, 
please refer to the vignette of this package by running 
\code{browseVignettes("signatureSearchData")} in R.
}

\references{
  LINCS source data at 5 levels from GEO: 
  https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92742
}

\examples{
library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "lincs_expr"))
# lincs_expr_path <- eh[["EH3227"]]
# rhdf5::h5ls(lincs_expr_path) 
}

\keyword{datasets}

\name{ES_NULL}
\alias{ES_NULL}
\title{Null Distribution of WTCS from \code{gess_lincs} Method} 
\description{
This pre-computed WTCS null distribution is generated with 1000 random queries
(150 randomly sampled up and down gene labels sets) searching against the 
\code{\link{lincs}} database. It is used for computing nominal 
p-values of WTCS scores in the \code{gess_lincs} result from 
\code{signatureSearch} package.
}

\references{
For detailed description of the \code{gess_lincs} method and scores, 
please refer to the vignette of the \code{signatureSearch} package by
running \code{browseVignettes("signatureSearch")} in your R session.
}

\seealso{
   \code{gess_lincs}
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
es_null <- eh[["EH3234"]]
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runLimma.R
\name{runLimma}
\alias{runLimma}
\title{DEG Analysis with Limma}
\usage{
runLimma(df, comp_list, fdr = 0.05, foldchange = 1, verbose = TRUE)
}
\arguments{
\item{df}{data.frame containing normalized intensity values of CMAP02 samples}

\item{comp_list}{list of CEL file ids of treatment and control samples for 
each compound treatment. The list for CMAP02 data is generated from 
the \code{\link{sampleList}} function.}

\item{fdr}{cutoff of false discovery rate (FDR) for defining DEGs}

\item{foldchange}{cutoff of log2 fold change (LFC) for defining DEGs}

\item{verbose}{TRUE or FALSE}
}
\value{
list containing DEGs and log2FC matrix
}
\description{
This function runs DEG analysis with Limma at user defined FDR and LFC
cutoff by providing CMAP02 normalized expression values and annotation of
treatment v.s. control samples. For more context of the CMAP02 database, 
please consult the vignette of this package.
}
\examples{
path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
cmap_inst <- read.delim(path, check.names=FALSE) 
comp_list <- sampleList(cmap_inst, myby="CMP_CELL")
df <- as.data.frame(matrix(runif(70), ncol=7))
colnames(df) <- unlist(comp_list[1])
degList <- runLimma(df, comp_list[1])  
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizeCel.R
\name{normalizeCel}
\alias{normalizeCel}
\title{Normalize CEL Files}
\usage{
normalizeCel(chiptype_list, batchsize = 300, rerun = TRUE)
}
\arguments{
\item{chiptype_list}{list, storing CEL files in each chiptype}

\item{batchsize}{number of CEL files to import and to normalize at once}

\item{rerun}{TRUE or FALSE, whether to run the function}
}
\value{
Files storing normalized expression values in matrix, here one matrix file 
per chip type
}
\description{
Function processes the CEL files from each chip type (three for CMap2 data)
separately using the MAS5 normalization algorithm. The results will be written
to subdirectores (under a data parent directory) that are named after the chip
type names. To reduce the memory consumption of this step, the CEL files are
processed in user definable batch sizes. For details on the overall workflow,
please consult the vignette of this package. The normalization takes about 10
hours without parallelization. To save time, this process can be easily
accelerated on a computer cluster.
}
\examples{
# chiptype_list <- split(names(chiptype), as.character(chiptype))
normalizeCel(chiptype_list, rerun=FALSE)
}

\name{goAnno}
\alias{goAnno}
\title{GO Term to Gene SYMBOLs Mapping Annotation} 
\description{
It is a \code{data.frame} containing the GO term ID to gene SYMBOLs mapping 
information used for \code{dtnetplot} function in \code{signatureSearch} package
to get the annotated gene labels set if the 'set' argument is a GO term ID.
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
goAnno <- eh["EH3229"]
# goAnno <- eh[["EH3229"]]
}
\keyword{datasets}

\name{taurefList}
\alias{taurefList}
\title{Lookup Table for Computing Tau Scores} 
\description{
This dataset is generated by searching all signatures in the LINCS database
against itself to obtain a background distribution of NCS values (Qref). The
latter is used to compute the Tau scores of the \code{gess_lincs} method in the
\code{signatureSearch} package. The Tau score compares the observed enrichment
score to all others in Qref. It represents the percentage of reference queries
with a lower |NCS| than |NCSq,r|, adjusted to retain the sign of NCSq,r.
NCSq,r is the normalized connectivity score for signature r relative to query
q. A tau of 90 indicates that only 10 percent of reference perturbations showed
stronger connectivity to the query. 
}

\references{
For a detailed description of the \code{gess_lincs} method and its scores, 
please refer to the vignette of the \code{signatureSearch} package by
running \code{browseVignettes("signatureSearch")}.

Subramanian, A., Narayan, R., Corsello, S. M., Peck, D. D., 
Natoli, T. E., Lu, X., Golub, T. R. (2017). A Next Generation 
Connectivity Map: L1000 Platform and the First 1,000,000 Profiles. Cell, 
171(6), 1437-1452.e17. https://doi.org/10.1016/j.cell.2017.10.049
}

\seealso{
    \code{gess_lincs}
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
taurefList <- eh["EH3233"]
# taurefList <- eh[["EH3233"]]
}

\keyword{datasets}

\name{cmap_rank}
\alias{cmap_rank}
\title{Cmap Rank Signature Database}
\description{
CMap2 (Version build02) contains GESs for 1,309 drugs and eight cell lines that 
were generated with Affymetrix Gene Chips as expression platform. In some cases 
this includes drug treatments at different concentrations and time points. For 
consistency, the CMap2 data was reduced to drug treatments with concentrations 
and time points that are comparable to those used for the LINCS data. CMap2 data 
can be downloaded from GEO or its project site either in raw format or as rank 
transformed matrix. The ranks are based on DEG analyses of drug treatments 
(drug vs. no-drug) where the resulting Z-scores were used to generate the 
rank matrix. The latter was used here and is referred to as rankMatrix. The 
Affymetrix probe set identifiers stored in the row name slot of this matrix 
were translated into gene identifies. To obtain a matrix with unique gene 
identifiers, the ranks for genes represented by more than one probe set were 
averaged and then re-ranked accordingly. This final gene level rank matrix,
referred to as \code{cmap_rank}, contains rank profiles for 12,403 genes from 
1,309 compound treatments in up to 5 cells corresponding to a total of 
3,587 treatment signatures. This matrix can be used for all GESS methods in 
the signatureSearch package that are compatible with rank data, such as the 
\code{gess_cmap} method.
}
\details{
The \code{cmap_rank} data can be downloaded from Bioconductor’s ExperimentHub as 
HDF5 file. Since CMap2 is much smaller than LINCS, it can be imported in its 
entirety into a SummarizedExperiment object without excessive memory requirements
as shown in the Examples section.

The loaded \code{cmap_rank} data object is generated from the \code{rankMatrix}
downloaded from the CMap project site.
For documentation and code of generating the cmap_rank databases from sources, 
please refer to the vignette of this package by running 
\code{browseVignettes("signatureSearchData")} in R.
}

\references{
CMap project site: https://portals.broadinstitute.org/cmap
}

\examples{
library(ExperimentHub)
# eh <- ExperimentHub()
# query(eh, c("signatureSearchData", "cmap_rank"))
# cmap_rank_path <- eh[["EH3225"]]
# rhdf5::h5ls(cmap_rank_path)
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/probe2gene.R
\name{probe2gene}
\alias{probe2gene}
\title{Transform Probe Set to Gene Level Data}
\usage{
probe2gene(expr_df, annot)
}
\arguments{
\item{expr_df}{data.frame with probe set level expression data}

\item{annot}{data.frame containing probe set id (in row name slot) to
gene entrez id (in first column) mapping information}
}
\value{
data.frame with gene level expression values
}
\description{
Function to transform expression values from probe set to gene level data. If
genes are represented by several probe sets then their mean intensities are used.
Probe sets not matching any gene ids are removed.
}
\examples{
expr_df <- data.frame(t1=runif(3), t2=runif(3), row.names=paste0("p", 1:3))
annot <- data.frame(ENTREZID=c("123","123","124"), 
                    SYMBOL=c("g1","g1","g2"),
                    row.names=paste0("p", 1:3))
gdf <- probe2gene(expr_df, annot)
}

\name{dtlink_db_clue_sti}
\alias{dtlink_db_clue_sti}
\title{Drug-Target Annotation Resources}
\description{
This is a SQLite database containing drug-target links (drug names to gene SYMBOL ids)
obtained from DrugBank, CLUE, and STITCH annotation databases. 
}
\details{
It is an intermediate database used for functionality in the 
\code{signatureSearch} software package to get target genes/proteins of query 
drugs in DrugBank (https://www.drugbank.ca/), CLUE (https://clue.io/) and STITCH
(http://stitch.embl.de/) databases.

The target proteins of drugs are mapped to their encoding genes by using 
identifier mapping resources from R/Bioconductor such as the \code{org.Hs.eg.db}
annotation package. Thus, the drug-target links in this database are drug names
to gene SYMBOLs mapping. 

The STITCH database provides confidence scores for each drug-target interaction.
Only interactions with confidence scores >=0.7 were selected.

The drug-target interaction annotations from the above three databases are combined.
To minimize noise, e.g. from promiscuous binders, drugs with more than 100 
distinct targets are removed. Similarly, targets with more than 100 annotated 
drugs are excluded.
}

\seealso{
    \code{get_targets}
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
qr <- query(eh, c("signatureSearchData", "dtlink_db_clue_sti"))
dtlink <- eh["EH3228"]
# dtlink <- eh[["EH3228"]]
}

\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_filter.R
\name{sig_filter}
\alias{sig_filter}
\title{Filter LINCS Level 5 by Condition}
\usage{
sig_filter(meta, pert_type = "trt_cp", dose, time = "24 h")
}
\arguments{
\item{meta}{tibble containing experimental conditions of LINCS data}

\item{pert_type}{perturbation type, 'trt_cp' refers to treatment ('trt') with compound ('cp'). 
Description of other perturbation types ('pert_type') can be found in the GEO CMap LINCS 
User Guide v2.1 URL: https://docs.google.com/document/d/1rbHBy3DKekFm9lZouRG-ZcfLmCsfkUKzGPxxjqxPlYw/edit#}

\item{dose}{concentration of compound used for treatment, needs to match elements
in 'pert_idose' column of 'meta'}

\item{time}{compound treatment time, needs to match elements in 'pert_itime' 
column of 'meta'}
}
\value{
tibble
}
\description{
Function to filter Level 5 data from LINCS by specific conditions, such as compound 
concentrations and treatment times.
}
\examples{
meta <- data.frame(pert_idose=c("2 um","4 um", "10 um"), 
                   pert_itime="24 h",
                   pert_type="trt_cp", pert_iname=c("p1","p2","p3"),
                   cell_id="MCF7")
sig_filter(meta, dose="2 um")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCmapCEL.R
\name{getCmapCEL}
\alias{getCmapCEL}
\title{Download CMap2 CEL Files}
\usage{
getCmapCEL(rerun = TRUE, dest_dir = tempdir())
}
\arguments{
\item{rerun}{TRUE or FALSE, whether to download the data}

\item{dest_dir}{character(1), path to the destination directory}
}
\value{
download files
}
\description{
This function will download the 7,056 CEL files from the CMap2 project 
site (http://www.broadinstitute.org/cmap), and save each of them to a
subdirectory named CEL under data. Since this download step will take 
a long time, the \code{rerun} argument has been assigned \code{FALSE} in
the example code below to avoid running it accidentally. If the raw data are not
needed, users can skip this time consuming step and work with the preprocessed
\code{\link{cmap}} or \code{\link{cmap_expr}} database downloaded from the 
ExperimentHub instead.
}
\examples{
getCmapCEL(rerun=FALSE) # set 'rerun' to TRUE if download
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meanExpr.R
\name{meanExpr}
\alias{meanExpr}
\title{Calculate Mean Value for Replicated Samples}
\usage{
meanExpr(expr_df, cmap_inst)
}
\arguments{
\item{expr_df}{\code{data.frame} containing normalized expression values}

\item{cmap_inst}{\code{data.frame} defining experimental conditions}
}
\value{
\code{data.frame} with mean expression values of replicates
}
\description{
Function averages the normalized gene expression values for different
concentrations, treatment times and replicates of compounds and cell types.
For more context on this step, please consult the corresponding section in the
vignette of this package.
}
\examples{
path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
cmap_inst <- read.delim(path, check.names=FALSE) 
expr_df <- as.data.frame(matrix(runif(30), ncol=5))
colnames(expr_df) <- paste0(cmap_inst$perturbation_scan_id[seq_len(5)],".CEL")
mexpr <- meanExpr(expr_df, cmap_inst[seq_len(5),])
}

\name{signatureSearchData}
\alias{signatureSearchData}
\title{signatureSearchData: Reference Data for Gene Expression Signature Searching}
\description{
This package provides access to the reference data used by the associated 
signatureSearch software package. The latter allows to search with a query 
gene expression signature (GES) a database of treatment GESs to identify 
cellular states sharing similar expression responses (connections). This way 
one can identify drugs or gene knockouts that induce expression phenotypes 
similar to a sample of interest. The resulting associations may lead to novel 
functional insights how perturbagens of interest interact with biological systems.

Currently, this data package includes GES data from the CMap (Connectivity Map) 
and LINCS (Library of Network-Based Cellular Signatures) projects that are 
largely based on drug and genetic perturbation experiments performed on 
variable numbers of human cell lines (Lamb et al. 2006; Subramanian et al. 2017).
In signatureSearchData these data sets have been preprocessed to be compatible 
with the different gene expression signature search (GESS) algorithms 
implemented in signatureSearch. The preprocessed data types include but are not 
limited to normalized gene expression values (e.g. intensity values), log fold 
changes (LFC) and Z-scores, p-values or FDRs of differentially expressed genes 
(DEGs), rankings based on selected preprocessing routines or sets of top 
up/down-regulated DEGs.

The CMap data were downloaded from the CMap project site (Version build02). 
The latter is a collection of over 7,000 gene expression profiles (signatures) 
obtained from perturbation experiments with 1,309 drug-like small molecules 
on five human cancer cell lines. The Affymetrix Gene Chip technology was used 
to generate the CMAP2 data set.

In 2017, the LINCS Consortium generated a similar but much larger data set 
where the total number of gene expression signatures was scaled up to over 
one million. This was achieved by switching to a much more cost effective gene 
expression profiling technology called L1000 assay (Peck et al. 2006; Edgar, 
Domrachev, and Lash 2002). The current set of perturbations covered by the 
LINCS data set includes 19,811 drug-like small molecules applied at variable 
concentrations and treatment times to ~70 human non-cancer (normal) and cancer 
cell lines. Additionally, it includes several thousand genetic perturbagens 
composed of gene knockdown and over-expression experiments.

The data structures and search algorithms used by signatureSearch and 
signatureSearchData are designed to work with most genome-wide expression data 
including hybridization-based methods, such as Affymetrix or L1000, as well as 
sequencing-based methods, such as RNA-Seq. Currently, signatureSearchData does 
not include preconfigured RNA-Seq reference data mainly due to the lack of 
large-scale perturbation studies (e.g. drug-based) available in the public 
domain that are based on RNA-Seq. This situation may change in the near future 
once the technology has become more affordable for this purpose.

This package also contains other intermediate annotation datasets, such as drug-target 
information of small molecules in DrugBank, CLUE and STITCH databases used for 
getting target annotation of query drugs, GO system annotations and drugs to GO 
functional category mappings used for signatureSearch's functional enrichment 
analysis (FEA) routines or null distribution of Enrichment 
Scores (ES) used for computing WTCS p-values from LINCS GESS method.
}
\details{
This package contains the following main datasets:
  \itemize{
    \item \code{\link{cmap}} 
    \item \code{\link{cmap_expr}} 
    \item \code{\link{cmap_rank}} 
    \item \code{\link{lincs}} 
    \item \code{\link{lincs_expr}} 
    \item \code{\link{dtlink_db_clue_sti}}
    \item \code{\link{taurefList}}
  }
For documentation of generating the CMAP/LINCS databases from sources, 
please refer to the vignette of this package by running 
\code{browseVignettes("signatureSearchData")} in R.
}

\seealso{
   \code{signatureSearch}
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
ssd <- query(eh, c("signatureSearchData"))
ssd
ssd$title
as.list(ssd)[10]

## Retrieve CMAP databases
cmap <- eh["EH3223"] # stores LFC scores
cmap_expr <- eh["EH3224"] # stores gene expression intensity values
cmap_rank <- eh["EH3225"] # stores gene ranks

## Retrieve LINCS databases
lincs <- eh['EH3226'] # stores moderated z-scores
lincs_expr <- eh['EH3227'] # stores gene expression intensity values
}

\references{
Lamb, Justin, Emily D Crawford, David Peck, Joshua W Modell, Irene C Blat, 
Matthew J Wrobel, Jim Lerner, et al. 2006. “The Connectivity Map: Using 
Gene-Expression Signatures to Connect Small Molecules, Genes, and Disease.” 
Science 313 (5795): 1929–35. http://dx.doi.org/10.1126/science.1132939.

Subramanian, Aravind, Rajiv Narayan, Steven M Corsello, David D Peck, Ted E 
Natoli, Xiaodong Lu, Joshua Gould, et al. 2017. “A Next Generation Connectivity 
Map: L1000 Platform and the First 1,000,000 Profiles.” Cell 171 (6): 
1437–1452.e17. http://dx.doi.org/10.1016/j.cell.2017.10.049.

Edgar, Ron, Michael Domrachev, and Alex E Lash. 2002. “Gene Expression Omnibus: 
NCBI Gene Expression and Hybridization Array Data Repository.” Nucleic Acids 
Res. 30 (1): 207–10. https://www.ncbi.nlm.nih.gov/pubmed/11752295.

Peck, David, Emily D Crawford, Kenneth N Ross, Kimberly Stegmaier, Todd R Golub,
and Justin Lamb. 2006. “A Method for High-Throughput Gene Expression Signature 
Analysis.” Genome Biol. 7 (7): R61. http://dx.doi.org/10.1186/gb-2006-7-7-r61.
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampleList.R
\name{sampleList}
\alias{sampleList}
\title{Generate List of Treatment vs. Control Samples}
\usage{
sampleList(cmap, myby)
}
\arguments{
\item{cmap}{data.frame containing experiment information of CMAP02 instances}

\item{myby}{"CMP" or "CMP_CELL", "CMP": by compound treatments in all cells;
"CMP_CELL": by compound treatments in individual cell}
}
\value{
a list object containing treatment samples and control samples ids
for each compound treament.
}
\description{
This function generates a list of treatment samples and control samples CEL ids
for each compound treatment from the experiment annotation of CMAP02 
instances. For more context of the CMAP02 database, please consult the vignette
of this package.
}
\examples{
path <- system.file("extdata", "cmap_instances_02.txt", package="signatureSearchData")
cmap_inst <- read.delim(path, check.names=FALSE) 
comp_list <- sampleList(cmap_inst, myby="CMP_CELL")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineResults.R
\name{combineResults}
\alias{combineResults}
\title{Combine Batch-processed Normalization Results}
\usage{
combineResults(chiptype_dir, rerun = TRUE)
}
\arguments{
\item{chiptype_dir}{character vector containing paths to chiptype directories}

\item{rerun}{TRUE or FALSE, whether to run the function}
}
\value{
File storing normalization values for each CEL file type
}
\description{
Function combines batch-processed normalization results from same chip type
into a single data frame. Typically, the input for this step is generated by
the upstream \code{normalizeCel} function. For more context, please consult
the vignette of this package.
}
\examples{
# chiptype_dir <- unique(readRDS("./data/chiptype.rds"))
combineResults(chiptype_dir, rerun=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_filter.R
\name{inst_filter}
\alias{inst_filter}
\title{Filter LINCS Level 3 by Condition}
\usage{
inst_filter(
  meta,
  pert_type = "trt_cp",
  dose = 10,
  dose_unit = "um",
  time = 24,
  time_unit = "h"
)
}
\arguments{
\item{meta}{tibble containing experimental conditions of LINCS data}

\item{pert_type}{perturbation type, 'trt_cp' refers to treatment ('trt') with 
compound ('cp'). Description of other perturbation types ('pert_type') can 
be found in the GEO CMap LINCS User Guide v2.1 
URL: https://docs.google.com/document/d/1rbHBy3DKekFm9lZouRG-ZcfLmCsfkUKzGPxxjqxPlYw/edit#}

\item{dose}{concentration of compound used for treatment, needs to match elements
in 'pert_dose' column of 'meta'}

\item{dose_unit}{unit of dose of compound treatment, needs to match elements
in 'pert_dose_unit' column of 'meta'}

\item{time}{compound treatment time, needs to match elements in 'pert_time' 
column of 'meta'}

\item{time_unit}{unit of time, needs to match elements in 'pert_time_unit' 
column of 'meta'}
}
\value{
tibble
}
\description{
Function to filter Level 3 data from LINCS by specific conditions, such as compound 
concentrations and treatment times.
}
\examples{
meta <- data.frame(pert_dose=c(2,4,10), pert_dose_unit="um", 
                   pert_time=24, pert_time_unit="h",
                   pert_type="trt_cp", pert_iname=c("p1","p2","p3"),
                   cell_id="MCF7")
inst_filter(meta)
}

\name{GO_DATA}
\alias{GO_DATA}
\title{GO Annotation Environment} 
\description{
It is an RDS file storing the GO annotation environment (GO term ID to 
gene SYMBOLs mapping, gene SYMBOL to annotated GO terms, GO term ID to 
its description, GO term ID to its ontology) 
used for \code{tsea_*} functions in \code{signatureSearch} package with GO 
enrichment analysis. This environment is saved to an RDS file to accelerate the 
analysis speed by avoiding building this environment from scratch every time 
running the \code{tsea} functions.
}

\seealso{
    \code{tsea}
}

\examples{
library(ExperimentHub)
eh <- ExperimentHub()
qr <- query(eh, c("signatureSearchData", "GO_DATA"))
go_data <- eh["EH3231"]
# go_data <- eh[["EH3231"]]
}

\keyword{datasets}
